@@ -2,15 +2,16 @@
 
 using namespace sirius;
 
-void test_allgather()
+void
+test_allgather()
 {
     int N = 11;
     std::vector<double> vec(N, 0.0);
 
-    sddk::splindex<sddk::splindex_t::block> spl(N, mpi::Communicator::world().size(), mpi::Communicator::world().rank());
+    splindex_block<> spl(N, n_blocks(mpi::Communicator::world().size()), block_id(mpi::Communicator::world().rank()));
 
     for (int i = 0; i < spl.local_size(); i++) {
-        vec[spl[i]] = mpi::Communicator::world().rank() + 1.0;
+        vec[spl.global_index(i)] = mpi::Communicator::world().rank() + 1.0;
     }
 
     {
@@ -40,7 +41,8 @@ void test_allgather()
     mpi::Communicator::world().barrier();
 }
 
-int main(int argn, char** argv)
+int
+main(int argn, char** argv)
 {
     sirius::initialize(true);
 
