@@ -1,19 +1,20 @@
 #include <sirius.hpp>
 
 using namespace sirius;
-using namespace sddk;
 
-int test1(int size)
+int
+test1(int size)
 {
-    double t = -utils::wtime();
-    sddk::mdarray<char, 1> buf(size);
-    sddk::mdarray<char, 1> buf1(size);
+    double t = -wtime();
+    mdarray<char, 1> buf({size});
+    mdarray<char, 1> buf1({size});
     buf.zero();
 
     for (int r = 0; r < mpi::Communicator::world().size(); r++) {
-        int rank = mpi::Communicator::world().rank();
+        int rank  = mpi::Communicator::world().rank();
         int rank1 = (rank + 1) % mpi::Communicator::world().size();
-        auto req = mpi::Communicator::world().isend(buf.at(memory_t::host), size, rank1, mpi::Communicator::get_tag(rank, rank1));
+        auto req  = mpi::Communicator::world().isend(buf.at(memory_t::host), size, rank1,
+                                                     mpi::Communicator::get_tag(rank, rank1));
         int rank2 = rank - 1;
         if (rank2 < 0) {
             rank2 = mpi::Communicator::world().size() - 1;
@@ -21,14 +22,15 @@ int test1(int size)
         mpi::Communicator::world().recv(buf1.at(memory_t::host), size, rank2, mpi::Communicator::get_tag(rank, rank2));
         req.wait();
     }
-    t += utils::wtime();
+    t += wtime();
     if (mpi::Communicator::world().rank() == 0) {
         printf("time : %f sec.\n", t);
     }
     return 0;
 }
 
-int main(int argn, char** argv)
+int
+main(int argn, char** argv)
 {
     cmd_args args;
     args.register_key("--size=", "buffer size in bytes");
