@@ -2,15 +2,15 @@
 #include "testing.hpp"
 
 using namespace sirius;
-using namespace sddk;
 
-int test1()
+int
+test1()
 {
     int lmax{20};
 
     SHT sht(device_t::CPU, lmax);
 
-    mdarray<std::complex<double>, 2> ylm(utils::lmmax(lmax), sht.num_points());
+    mdarray<std::complex<double>, 2> ylm({sf::lmmax(lmax), sht.num_points()});
     for (int i = 0; i < sht.num_points(); i++) {
         sf::spherical_harmonics(lmax, sht.theta(i), sht.phi(i), &ylm(0, i));
     }
@@ -25,8 +25,8 @@ int test1()
                         for (int m3 = -l3; m3 <= l3; m3++) {
                             std::complex<double> s{0};
                             for (int i = 0; i < sht.num_points(); i++) {
-                                s += std::conj(ylm(utils::lm(l1, m1), i)) * ylm(utils::lm(l2, m2), i) *
-                                     ylm(utils::lm(l3, m3), i) * sht.weight(i);
+                                s += std::conj(ylm(sf::lm(l1, m1), i)) * ylm(sf::lm(l2, m2), i) *
+                                     ylm(sf::lm(l3, m3), i) * sht.weight(i);
                             }
                             s *= fourpi;
                             d += std::abs(s.real() - SHT::gaunt_yyy(l1, l2, l3, m1, m2, m3));
@@ -43,7 +43,8 @@ int test1()
     }
 }
 
-int main(int argn, char** argv)
+int
+main(int argn, char** argv)
 {
     int err{0};
     err += call_test("<Ylm|Ylm|Ylm> numerical", test1);
