@@ -4,12 +4,13 @@
 
 using namespace sirius;
 
-int run_test(cmd_args& args)
+int
+run_test(cmd_args& args)
 {
     int n{10};
 
     for (int i = 0; i < 20; i++) {
-        double phi = utils::random<double>() * fourpi;
+        double phi = random<double>() * fourpi;
         auto cosxn = sf::cosxn(n, phi);
         auto sinxn = sf::sinxn(n, phi);
         for (int l = 0; l < n; l++) {
@@ -24,7 +25,8 @@ int run_test(cmd_args& args)
     return 0;
 }
 
-int main(int argn, char** argv)
+int
+main(int argn, char** argv)
 {
     cmd_args args;
 
@@ -34,9 +36,15 @@ int main(int argn, char** argv)
     printf("running %-30s : ", argv[0]);
     int result = run_test(args);
     if (result) {
-        printf("\x1b[31m" "Failed" "\x1b[0m" "\n");
+        printf("\x1b[31m"
+               "Failed"
+               "\x1b[0m"
+               "\n");
     } else {
-        printf("\x1b[32m" "OK" "\x1b[0m" "\n");
+        printf("\x1b[32m"
+               "OK"
+               "\x1b[0m"
+               "\n");
     }
     sirius::finalize();
 
