@@ -4,9 +4,9 @@ extern "C" {
 }
 
 using namespace sirius;
-using namespace sddk;
 
-void create_supercell(cmd_args const& args__)
+void
+create_supercell(cmd_args const& args__)
 {
     r3::matrix<int> scell;
     std::stringstream s(args__.value<std::string>("supercell"));
@@ -60,7 +60,7 @@ void create_supercell(cmd_args const& args__)
         auto label = ctx.unit_cell().atom_type(iat).label();
 
         for (int i = 0; i < ctx.unit_cell().atom_type(iat).num_atoms(); i++) {
-            int ia = ctx.unit_cell().atom_type(iat).atom_id(i);
+            int ia  = ctx.unit_cell().atom_type(iat).atom_id(i);
             auto va = ctx.unit_cell().atom(ia).position();
 
             for (int i0 = -10; i0 <= 10; i0++) {
@@ -71,8 +71,8 @@ void create_supercell(cmd_args const& args__)
                         r3::vector<double> vf = ctx_sc.unit_cell().get_fractional_coordinates(vc);
 
                         auto vr = reduce_coordinates(vf);
-                        for (int x: {0, 1, 2}) {
-                            vr.first[x] = utils::round(vr.first[x], 10);
+                        for (int x : {0, 1, 2}) {
+                            vr.first[x] = round(vr.first[x], 10);
                         }
                         bool add_atom = (ctx_sc.unit_cell().atom_id_by_position(vr.first) == -1);
                         //==if (add_atom && iat == 2)
@@ -91,8 +91,8 @@ void create_supercell(cmd_args const& args__)
     }
     std::printf("number of atoms in the supercell: %i\n", ctx_sc.unit_cell().num_atoms());
 
-    //ctx_sc.unit_cell().get_symmetry();
-    //ctx_sc.unit_cell().print_info(4);
+    // ctx_sc.unit_cell().get_symmetry();
+    // ctx_sc.unit_cell().print_info(4);
     ctx_sc.unit_cell().write_cif();
     json dict;
     dict["unit_cell"] = ctx_sc.unit_cell().serialize();
@@ -102,29 +102,30 @@ void create_supercell(cmd_args const& args__)
     }
 }
 
-void find_primitive()
+void
+find_primitive()
 {
     Simulation_context ctx("sirius.json", mpi::Communicator::self());
 
     double lattice[3][3];
-    for (int i: {0, 1, 2}) {
-        for (int j: {0, 1, 2}) {
+    for (int i : {0, 1, 2}) {
+        for (int j : {0, 1, 2}) {
             lattice[i][j] = ctx.unit_cell().lattice_vector(j)[i];
         }
     }
-    mdarray<double, 2> positions(3, 4 * ctx.unit_cell().num_atoms());
-    mdarray<int, 1> types(4 * ctx.unit_cell().num_atoms());
+    mdarray<double, 2> positions({3, 4 * ctx.unit_cell().num_atoms()});
+    mdarray<int, 1> types({4 * ctx.unit_cell().num_atoms()});
     for (int ia = 0; ia < ctx.unit_cell().num_atoms(); ia++) {
-        for (int x: {0, 1, 2}) {
+        for (int x : {0, 1, 2}) {
             positions(x, ia) = ctx.unit_cell().atom(ia).position()[x];
-            types(ia) = ctx.unit_cell().atom(ia).type_id();
+            types(ia)        = ctx.unit_cell().atom(ia).type_id();
         }
     }
 
     std::printf("original number of atoms: %i\n", ctx.unit_cell().num_atoms());
 
-    int nat_new = spg_standardize_cell(lattice, (double(*)[3])&positions(0, 0), &types[0], ctx.unit_cell().num_atoms(),
-                                       1, 0, ctx.cfg().control().spglib_tolerance());
+    int nat_new = spg_standardize_cell(lattice, (double(*)[3]) & positions(0, 0), &types[0],
+                                       ctx.unit_cell().num_atoms(), 1, 0, ctx.cfg().control().spglib_tolerance());
     std::printf("new number of atoms: %i\n", nat_new);
 
     Simulation_context ctx_new(mpi::Communicator::self());
@@ -137,8 +138,7 @@ void find_primitive()
     }
     ctx_new.unit_cell().set_lattice_vectors(a0, a1, a2);
 
-    for (int iat = 0; iat < ctx.unit_cell().num_atom_types(); iat++)
-    {
+    for (int iat = 0; iat < ctx.unit_cell().num_atom_types(); iat++) {
         auto label = ctx.unit_cell().atom_type(iat).label();
         ctx_new.unit_cell().add_atom_type(label, "");
     }
@@ -161,43 +161,44 @@ void find_primitive()
         std::ofstream ofs("unit_cell.json", std::ofstream::out | std::ofstream::trunc);
         ofs << dict.dump(4);
     }
-
 }
 
-void create_qe_input(cmd_args const& args__)
+void
+create_qe_input(cmd_args const& args__)
 {
     Simulation_context ctx(args__.value<std::string>("input", "sirius.json"), mpi::Communicator::self());
 
     FILE* fout = fopen("pw.in", "w");
     fprintf(fout, "&control\n"
-    "calculation=\'scf\',\n"
-    "restart_mode=\'from_scratch\',\n"
-    "pseudo_dir = \'./\',\n"
-    "outdir=\'./\',\n"
-    "prefix = \'scf_\',\n"
-    "tstress = false,\n"
-    "tprnfor = false,\n"
-    "verbosity = \'high\',\n"
-    "disk_io = \'none\',\n"
-    "wf_collect = false\n"
-    "/\n");
+                  "calculation=\'scf\',\n"
+                  "restart_mode=\'from_scratch\',\n"
+                  "pseudo_dir = \'./\',\n"
+                  "outdir=\'./\',\n"
+                  "prefix = \'scf_\',\n"
+                  "tstress = false,\n"
+                  "tprnfor = false,\n"
+                  "verbosity = \'high\',\n"
+                  "disk_io = \'none\',\n"
+                  "wf_collect = false\n"
+                  "/\n");
 
-    fprintf(fout, "&system\nibrav=0, celldm(1)=1, ecutwfc=40, ecutrho = 300,\noccupations = \'smearing\', smearing = \'gauss\', degauss = 0.001,\n");
+    fprintf(fout, "&system\nibrav=0, celldm(1)=1, ecutwfc=40, ecutrho = 300,\noccupations = \'smearing\', smearing = "
+                  "\'gauss\', degauss = 0.001,\n");
     fprintf(fout, "nat=%i, ntyp=%i\n/\n", ctx.unit_cell().num_atoms(), ctx.unit_cell().num_atom_types());
     fprintf(fout, "&electrons\nconv_thr =  1.0d-11,\nmixing_beta = 0.7,\nelectron_maxstep = 100\n/\n");
     fprintf(fout, "&IONS\n"
-    "ion_dynamics=\'bfgs\',\n"
-    "/\n"
-    "&CELL\n"
-    "cell_dynamics=\'bfgs\',\n"
-    "/\n");
+                  "ion_dynamics=\'bfgs\',\n"
+                  "/\n"
+                  "&CELL\n"
+                  "cell_dynamics=\'bfgs\',\n"
+                  "/\n");
 
     fprintf(fout, "ATOMIC_SPECIES\n");
     for (int iat = 0; iat < ctx.unit_cell().num_atom_types(); iat++) {
         fprintf(fout, "%s 0.0 pp.UPF\n", ctx.unit_cell().atom_type(iat).label().c_str());
     }
 
-    fprintf(fout,"CELL_PARAMETERS\n");
+    fprintf(fout, "CELL_PARAMETERS\n");
     for (int i = 0; i < 3; i++) {
         for (int x = 0; x < 3; x++) {
             fprintf(fout, "%18.8f", ctx.unit_cell().lattice_vector(i)[x]);
@@ -209,8 +210,7 @@ void create_qe_input(cmd_args const& args__)
         for (int ia = 0; ia < ctx.unit_cell().atom_type(iat).num_atoms(); ia++) {
             int id = ctx.unit_cell().atom_type(iat).atom_id(ia);
             fprintf(fout, "%s  %18.8f %18.8f %18.8f\n", ctx.unit_cell().atom_type(iat).label().c_str(),
-                    ctx.unit_cell().atom(id).position()[0],
-                    ctx.unit_cell().atom(id).position()[1],
+                    ctx.unit_cell().atom(id).position()[0], ctx.unit_cell().atom(id).position()[1],
                     ctx.unit_cell().atom(id).position()[2]);
         }
     }
@@ -219,7 +219,8 @@ void create_qe_input(cmd_args const& args__)
     fclose(fout);
 }
 
-void create_exciting_input(cmd_args const& args__)
+void
+create_exciting_input(cmd_args const& args__)
 {
     Simulation_context ctx(args__.value<std::string>("input", "sirius.json"), mpi::Communicator::self());
 
@@ -235,7 +236,8 @@ void create_exciting_input(cmd_args const& args__)
     }
     fprintf(fout, "    </crystal>\n");
     for (int iat = 0; iat < ctx.unit_cell().num_atom_types(); iat++) {
-        fprintf(fout, "    <species speciesfile=\"%s.xml\" rmt=\"2.0\">\n", ctx.unit_cell().atom_type(iat).label().c_str());
+        fprintf(fout, "    <species speciesfile=\"%s.xml\" rmt=\"2.0\">\n",
+                ctx.unit_cell().atom_type(iat).label().c_str());
         for (int ia = 0; ia < ctx.unit_cell().atom_type(iat).num_atoms(); ia++) {
             int id = ctx.unit_cell().atom_type(iat).atom_id(ia);
             auto v = ctx.unit_cell().atom(id).position();
@@ -245,9 +247,11 @@ void create_exciting_input(cmd_args const& args__)
     }
 
     fprintf(fout, "  </structure>\n");
-    fprintf(fout, "  <groundstate do=\"fromscratch\" ngridk=\"2 2 2\" rgkmax=\"4.0\" gmaxvr=\"16\" maxscl=\"2\"  kptgroups=\"1\">\n");
+    fprintf(fout, "  <groundstate do=\"fromscratch\" ngridk=\"2 2 2\" rgkmax=\"4.0\" gmaxvr=\"16\" maxscl=\"2\"  "
+                  "kptgroups=\"1\">\n");
     fprintf(fout, "    <libxc exchange=\"XC_LDA_X\" correlation=\"XC_LDA_C_PZ\"/>\n");
-    fprintf(fout, "    <sirius densityinit=\"true\" density=\"true\" vha=\"true\" xc=\"true\" eigenstates=\"true\" sfacg=\"true\" cfun=\"true\"/>\n");
+    fprintf(fout, "    <sirius densityinit=\"true\" density=\"true\" vha=\"true\" xc=\"true\" eigenstates=\"true\" "
+                  "sfacg=\"true\" cfun=\"true\"/>\n");
     fprintf(fout, "    <spin/>\n");
     fprintf(fout, "  </groundstate>\n");
 
@@ -255,40 +259,41 @@ void create_exciting_input(cmd_args const& args__)
     fclose(fout);
 }
 
-void convert_to_mol(cmd_args& args__)
+void
+convert_to_mol(cmd_args& args__)
 {
     Simulation_context ctx(args__.value<std::string>("input", "sirius.json"), mpi::Communicator::self());
 
     json dict;
-    dict["unit_cell"] = ctx.unit_cell().serialize(true);
+    dict["unit_cell"]                          = ctx.unit_cell().serialize(true);
     dict["unit_cell"]["atom_coordinate_units"] = "au";
     if (mpi::Communicator::world().rank() == 0) {
         std::ofstream ofs("unit_cell.json", std::ofstream::out | std::ofstream::trunc);
         ofs << dict.dump(4);
     }
-
 }
 
-void scale_lattice(cmd_args& args__)
+void
+scale_lattice(cmd_args& args__)
 {
     Simulation_context ctx(args__.value<std::string>("input", "sirius.json"), mpi::Communicator::self());
-    //ctx.unit_cell().get_symmetry();
-    //ctx.unit_cell().print_symmetry_info(5);
+    // ctx.unit_cell().get_symmetry();
+    // ctx.unit_cell().print_symmetry_info(5);
 
     double scale = args__.value<double>("scale", 1.0);
 
     json dict;
     dict["unit_cell"] = ctx.unit_cell().serialize();
-    for (auto& x: dict["unit_cell"]["lattice_vectors"]) {
-        for (auto& y: x) {
+    for (auto& x : dict["unit_cell"]["lattice_vectors"]) {
+        for (auto& y : x) {
             double v = y.get<double>();
             v *= scale;
             y = v;
         }
     }
-    for (auto& coord: dict["unit_cell"]["atoms"]) {
+    for (auto& coord : dict["unit_cell"]["atoms"]) {
         for (size_t i = 0; i < coord.size(); i++) {
-            for (int x: {0, 1, 2}) {
+            for (int x : {0, 1, 2}) {
                 double v = coord[i][x].get<double>();
                 if (v > 0.5) {
                     v -= 1;
@@ -304,7 +309,8 @@ void scale_lattice(cmd_args& args__)
     }
 }
 
-int main(int argn, char** argv)
+int
+main(int argn, char** argv)
 {
     cmd_args args;
     args.register_key("--input=", "{string} input file name");
