@@ -4,16 +4,18 @@
 #include <pybind11/operators.h>
 #include <iostream>
 
-#include "mpi/communicator.hpp"
+#include "core/mpi/communicator.hpp"
+#include "core/rte/rte.hpp"
 
 namespace sirius {
 
-inline auto make_sirius_comm(pybind11::object py_comm)
+inline auto
+make_sirius_comm(pybind11::object py_comm)
 {
     PyObject* py_obj = py_comm.ptr();
-    MPI_Comm* comm_p  = PyMPIComm_Get(py_obj);
-    if(comm_p == NULL) {
-        throw std::runtime_error("invalid MPI_Comm object passed");
+    MPI_Comm* comm_p = PyMPIComm_Get(py_obj);
+    if (comm_p == NULL) {
+        RTE_THROW("invalid MPI_Comm object passed");
     }
     int rank = -1;
     MPI_Comm_rank(*comm_p, &rank);
@@ -22,7 +24,9 @@ inline auto make_sirius_comm(pybind11::object py_comm)
     return sirius_comm;
 }
 
-inline pybind11::handle make_pycomm(const mpi::Communicator& comm) {
+inline pybind11::handle
+make_pycomm(const mpi::Communicator& comm)
+{
     return pybind11::handle(PyMPIComm_New(comm.native()));
 }
 
