@@ -24,6 +24,7 @@
 
 #include "occupation_matrix.hpp"
 #include "symmetry/crystal_symmetry.hpp"
+#include "symmetry/symmetrize_occupation_matrix.hpp"
 
 namespace sirius {
 
@@ -49,7 +50,7 @@ Occupation_matrix::Occupation_matrix(Simulation_context& ctx__)
             auto Ttot = sym[isym].spg_op.inv_sym_atom_T[ja] - sym[isym].spg_op.inv_sym_atom_T[ia] +
                         dot(sym[isym].spg_op.invR, r3::vector<int>(T));
             if (!occ_mtrx_T_.count(Ttot)) {
-                occ_mtrx_T_[Ttot] = sddk::mdarray<std::complex<double>, 3>(nhwf, nhwf, ctx_.num_mag_comp());
+                occ_mtrx_T_[Ttot] = mdarray<std::complex<double>, 3>({nhwf, nhwf, ctx_.num_mag_comp()});
                 occ_mtrx_T_[Ttot].zero();
             }
         }
@@ -66,12 +67,12 @@ Occupation_matrix::add_k_point_contribution(K_point<T>& kp__)
 
     PROFILE("sirius::Occupation_matrix::add_k_point_contribution");
 
-    sddk::memory_t mem_host{sddk::memory_t::host};
-    sddk::memory_t mem{sddk::memory_t::host};
+    memory_t mem_host{memory_t::host};
+    memory_t mem{memory_t::host};
     la::lib_t la{la::lib_t::blas};
-    if (ctx_.processing_unit() == sddk::device_t::GPU) {
-        mem      = sddk::memory_t::device;
-        mem_host = sddk::memory_t::host_pinned;
+    if (ctx_.processing_unit() == device_t::GPU) {
+        mem      = memory_t::device;
+        mem_host = memory_t::host_pinned;
         la       = la::lib_t::gpublas;
     }
 
@@ -80,12 +81,12 @@ Occupation_matrix::add_k_point_contribution(K_point<T>& kp__)
 
     int nwfu = r.first;
 
-    sddk::matrix<std::complex<T>> occ_mtrx(nwfu, nwfu, get_memory_pool(sddk::memory_t::host), "occ_mtrx");
+    matrix<std::complex<T>> occ_mtrx({nwfu, nwfu}, get_memory_pool(memory_t::host), mdarray_label("occ_mtrx"));
     if (is_device_memory(mem)) {
         occ_mtrx.allocate(get_memory_pool(mem));
     }
 
-    // TODO collnear and non-collinear cases have a lot of similar code; there should be a way to combine it
+    // TODO colinear and non-collinear cases have a lot of similar code; there should be a way to combine it
 
     /* full non collinear magnetism */
     if (ctx_.num_mag_dims() == 3) {
@@ -94,8 +95,8 @@ Occupation_matrix::add_k_point_contribution(K_point<T>& kp__)
             dm.allocate(get_memory_pool(mem));
         }
         wf::inner(ctx_.spla_context(), mem, wf::spin_range(0, 2), kp__.spinor_wave_functions(),
-                wf::band_range(0, kp__.num_occupied_bands()), kp__.hubbard_wave_functions_S(),
-                wf::band_range(0, nwfu), dm, 0, 0);
+                  wf::band_range(0, kp__.num_occupied_bands()), kp__.hubbard_wave_functions_S(),
+                  wf::band_range(0, nwfu), dm, 0, 0);
 
         la::dmatrix<std::complex<T>> dm1(kp__.num_occupied_bands(), nwfu, get_memory_pool(mem_host), "dm1");
         #pragma omp parallel for
@@ -111,9 +112,9 @@ Occupation_matrix::add_k_point_contribution(K_point<T>& kp__)
         /* now compute O_{ij}^{sigma,sigma'} = \sum_{nk} <psi_nk|phi_{i,sigma}><phi_{j,sigma^'}|psi_nk> f_{nk} */
         auto alpha = std::complex<T>(kp__.weight(), 0.0);
         la::wrap(la).gemm('C', 'N', nwfu, nwfu, kp__.num_occupied_bands(), &alpha, dm.at(mem), dm.ld(), dm1.at(mem),
-                        dm1.ld(), &la::constant<std::complex<T>>::zero(), occ_mtrx.at(mem), occ_mtrx.ld());
+                          dm1.ld(), &la::constant<std::complex<T>>::zero(), occ_mtrx.at(mem), occ_mtrx.ld());
         if (is_device_memory(mem)) {
-            occ_mtrx.copy_to(sddk::memory_t::host);
+            occ_mtrx.copy_to(memory_t::host);
         }
 
         #pragma omp parallel for schedule(static)
@@ -129,14 +130,14 @@ Occupation_matrix::add_k_point_contribution(K_point<T>& kp__)
                    relativistic wave functions have different total angular
                    momentum for the same n
                 */
-                int s_idx[2][2] = {{0, 3}, {2, 1}};
+                int s_idx[2][2]    = {{0, 3}, {2, 1}};
                 const int lmmax_at = 2 * atom.type().lo_descriptor_hub(atomic_orbitals_[at_lvl].second).l() + 1;
                 for (int s1 = 0; s1 < ctx_.num_spins(); s1++) {
                     for (int s2 = 0; s2 < ctx_.num_spins(); s2++) {
                         for (int mp = 0; mp < lmmax_at; mp++) {
                             for (int m = 0; m < lmmax_at; m++) {
-                                local_[at_lvl](m, mp, s_idx[s1][s2]) +=
-                                    occ_mtrx(r.first * s1 + offset_[at_lvl] + m, r.first * s2 + offset_[at_lvl] + mp);
+                                local_[at_lvl](m, mp, s_idx[s1][s2]) += occ_mtrx(r.first * s1 + offset_[at_lvl] + m,
+                                                                                 r.first * s2 + offset_[at_lvl] + mp);
                             }
                         }
                     }
@@ -149,14 +150,17 @@ Occupation_matrix::add_k_point_contribution(K_point<T>& kp__)
            have two. The inner product takes care of this case internally. */
 
         for (int ispn = 0; ispn < ctx_.num_spins(); ispn++) {
+            if (kp__.num_occupied_bands(ispn) == 0) {
+                continue;
+            }
             la::dmatrix<std::complex<T>> dm(kp__.num_occupied_bands(ispn), nwfu, get_memory_pool(mem_host), "dm");
             if (is_device_memory(mem)) {
                 dm.allocate(get_memory_pool(mem));
             }
             /* compute <psi | phi> where |phi> are the Hubbard WFs */
             wf::inner(ctx_.spla_context(), mem, wf::spin_range(ispn), kp__.spinor_wave_functions(),
-                    wf::band_range(0, kp__.num_occupied_bands(ispn)), kp__.hubbard_wave_functions_S(),
-                    wf::band_range(0, nwfu), dm, 0, 0);
+                      wf::band_range(0, kp__.num_occupied_bands(ispn)), kp__.hubbard_wave_functions_S(),
+                      wf::band_range(0, nwfu), dm, 0, 0);
 
             la::dmatrix<std::complex<T>> dm1(kp__.num_occupied_bands(ispn), nwfu, get_memory_pool(mem_host), "dm1");
             #pragma omp parallel for
@@ -175,10 +179,10 @@ Occupation_matrix::add_k_point_contribution(K_point<T>& kp__)
              * calculation of the hubbard potential */
             auto alpha = std::complex<T>(kp__.weight() / ctx_.max_occupancy(), 0.0);
             la::wrap(la).gemm('C', 'N', nwfu, nwfu, kp__.num_occupied_bands(ispn), &alpha, dm.at(mem), dm.ld(),
-                            dm1.at(mem), dm1.ld(), &la::constant<std::complex<T>>::zero(), occ_mtrx.at(mem),
-                            occ_mtrx.ld());
+                              dm1.at(mem), dm1.ld(), &la::constant<std::complex<T>>::zero(), occ_mtrx.at(mem),
+                              occ_mtrx.ld());
             if (is_device_memory(mem)) {
-                occ_mtrx.copy_to(sddk::memory_t::host);
+                occ_mtrx.copy_to(memory_t::host);
             }
 
             #pragma omp parallel for
@@ -205,7 +209,8 @@ Occupation_matrix::add_k_point_contribution(K_point<T>& kp__)
                 auto z1 = std::exp(std::complex<double>(0, -twopi * dot(e.first, kp__.vk())));
                 for (int i = 0; i < nwfu; i++) {
                     for (int j = 0; j < nwfu; j++) {
-                        e.second(i, j, ispn) += static_cast<std::complex<T>>(occ_mtrx(i, j)) * static_cast<std::complex<T>>(z1);
+                        e.second(i, j, ispn) +=
+                                static_cast<std::complex<T>>(occ_mtrx(i, j)) * static_cast<std::complex<T>>(z1);
                     }
                 }
             }
@@ -213,227 +218,13 @@ Occupation_matrix::add_k_point_contribution(K_point<T>& kp__)
     }
 }
 
-template void Occupation_matrix::add_k_point_contribution<double>(K_point<double>& kp__);
-#ifdef USE_FP32
-template void Occupation_matrix::add_k_point_contribution<float>(K_point<float>& kp__);
+template void
+Occupation_matrix::add_k_point_contribution<double>(K_point<double>& kp__);
+#ifdef SIRIUS_USE_FP32
+template void
+Occupation_matrix::add_k_point_contribution<float>(K_point<float>& kp__);
 #endif
 
-void
-Occupation_matrix::symmetrize()
-{
-    if (!ctx_.hubbard_correction()) {
-        return;
-    }
-
-    auto& sym      = ctx_.unit_cell().symmetry();
-    const double f = 1.0 / sym.size();
-    std::vector<sddk::mdarray<std::complex<double>, 3>> local_tmp;
-
-    local_tmp.resize(local_.size());
-
-    for (int at_lvl = 0; at_lvl < static_cast<int>(local_.size()); at_lvl++) {
-        const int ia     = atomic_orbitals_[at_lvl].first;
-        auto const& atom = ctx_.unit_cell().atom(ia);
-        // We can skip the symmetrization for this atomic level since it does not contribute to the Hubbard correction
-        // (or U = 0)
-        if (atom.type().lo_descriptor_hub(atomic_orbitals_[at_lvl].second).use_for_calculation()) {
-            local_tmp[at_lvl] = sddk::mdarray<std::complex<double>, 3>(local_[at_lvl].size(0), local_[at_lvl].size(1), 4);
-            // TODO: use std::copy
-            memcpy(local_tmp[at_lvl].at(sddk::memory_t::host), local_[at_lvl].at(sddk::memory_t::host),
-                   sizeof(std::complex<double>) * local_[at_lvl].size());
-        } else {
-            local_tmp[at_lvl].zero();
-        }
-    }
-
-    for (int at_lvl = 0; at_lvl < static_cast<int>(local_.size()); at_lvl++) {
-        const int ia     = atomic_orbitals_[at_lvl].first;
-        const auto& atom = ctx_.unit_cell().atom(ia);
-        local_[at_lvl].zero();
-        // we can skip the symmetrization for this atomic level since it does not contribute to the Hubbard correction
-        // (or U = 0)
-        if (atom.type().lo_descriptor_hub(atomic_orbitals_[at_lvl].second).use_for_calculation()) {
-            int il             = atom.type().lo_descriptor_hub(atomic_orbitals_[at_lvl].second).l();
-            const int lmmax_at = 2 * il + 1;
-            // local_[at_lvl].zero();
-            sddk::mdarray<std::complex<double>, 3> dm_ia(lmmax_at, lmmax_at, 4);
-            for (int isym = 0; isym < sym.size(); isym++) {
-                int pr            = sym[isym].spg_op.proper;
-                auto eang         = sym[isym].spg_op.euler_angles;
-                auto rotm         = sht::rotation_matrix<double>(4, eang, pr);
-                auto spin_rot_su2 = rotation_matrix_su2(sym[isym].spin_rotation);
-
-                int iap = sym[isym].spg_op.inv_sym_atom[ia];
-                dm_ia.zero();
-
-                int at_lvl1 =
-                    find_orbital_index(iap, atom.type().lo_descriptor_hub(atomic_orbitals_[at_lvl].second).n(),
-                                       atom.type().lo_descriptor_hub(atomic_orbitals_[at_lvl].second).l());
-
-                for (int ispn = 0; ispn < ((ctx_.num_mag_dims() == 3) ? (4) : ctx_.num_spins()); ispn++) {
-                    for (int m1 = 0; m1 < lmmax_at; m1++) {
-                        for (int m2 = 0; m2 < lmmax_at; m2++) {
-                            for (int m1p = 0; m1p < lmmax_at; m1p++) {
-                                for (int m2p = 0; m2p < lmmax_at; m2p++) {
-                                    dm_ia(m1, m2, ispn) +=
-                                        rotm[il](m1, m1p) * rotm[il](m2, m2p) * local_tmp[at_lvl1](m1p, m2p, ispn) * f;
-                                }
-                            }
-                        }
-                    }
-                }
-
-                if (ctx_.num_mag_dims() == 0) {
-                    for (int m1 = 0; m1 < lmmax_at; m1++) {
-                        for (int m2 = 0; m2 < lmmax_at; m2++) {
-                            local_[at_lvl](m1, m2, 0) += dm_ia(m1, m2, 0);
-                        }
-                    }
-                }
-
-                if (ctx_.num_mag_dims() == 1) {
-                    int const map_s[3][2] = {{0, 0}, {1, 1}, {0, 1}};
-                    for (int j = 0; j < 2; j++) {
-                        int s1 = map_s[j][0];
-                        int s2 = map_s[j][1];
-
-                        for (int m1 = 0; m1 < lmmax_at; m1++) {
-                            for (int m2 = 0; m2 < lmmax_at; m2++) {
-                                std::complex<double> dm[2][2] = {{dm_ia(m1, m2, 0), 0}, {0, dm_ia(m1, m2, 1)}};
-
-                                for (int s1p = 0; s1p < 2; s1p++) {
-                                    for (int s2p = 0; s2p < 2; s2p++) {
-                                        local_[at_lvl](m1, m2, j) +=
-                                            dm[s1p][s2p] * spin_rot_su2(s1, s1p) * std::conj(spin_rot_su2(s2, s2p));
-                                    }
-                                }
-                            }
-                        }
-                    }
-                }
-
-                if (ctx_.num_mag_dims() == 3) {
-                    int s_idx[2][2] = {{0, 3}, {2, 1}};
-                    for (int m1 = 0; m1 < lmmax_at; m1++) {
-                        for (int m2 = 0; m2 < lmmax_at; m2++) {
-
-                            std::complex<double> dm[2][2];
-                            std::complex<double> dm1[2][2] = {{0.0, 0.0}, {0.0, 0.0}};
-                            for (int s1 = 0; s1 < ctx_.num_spins(); s1++) {
-                                for (int s2 = 0; s2 < ctx_.num_spins(); s2++) {
-                                    dm[s1][s2] = dm_ia(m1, m2, s_idx[s1][s2]);
-                                }
-                            }
-
-                            for (int i = 0; i < 2; i++) {
-                                for (int j = 0; j < 2; j++) {
-                                    for (int s1p = 0; s1p < 2; s1p++) {
-                                        for (int s2p = 0; s2p < 2; s2p++) {
-                                            dm1[i][j] +=
-                                                dm[s1p][s2p] * spin_rot_su2(i, s1p) * std::conj(spin_rot_su2(j, s2p));
-                                        }
-                                    }
-                                }
-                            }
-
-                            for (int s1 = 0; s1 < ctx_.num_spins(); s1++) {
-                                for (int s2 = 0; s2 < ctx_.num_spins(); s2++) {
-                                    local_[at_lvl](m1, m2, s_idx[s1][s2]) += dm1[s1][s2];
-                                }
-                            }
-                        }
-                    }
-                }
-            }
-        }
-    }
-
-    if (ctx_.cfg().hubbard().nonlocal().size() && ctx_.num_mag_dims() == 3) {
-        RTE_THROW("non-collinear nonlocal occupancy symmetrization is not implemented");
-    }
-
-    /* a pair of "total number, offests" for the Hubbard orbitals indexing */
-    auto r = ctx_.unit_cell().num_hubbard_wf();
-
-    for (int i = 0; i < static_cast<int>(ctx_.cfg().hubbard().nonlocal().size()); i++) {
-        auto nl = ctx_.cfg().hubbard().nonlocal(i);
-        int ia  = nl.atom_pair()[0];
-        int ja  = nl.atom_pair()[1];
-        int il  = nl.l()[0];
-        int jl  = nl.l()[1];
-        int n1  = nl.n()[0];
-        int n2  = nl.n()[1];
-        int ib  = 2 * il + 1;
-        int jb  = 2 * jl + 1;
-        auto T  = nl.T();
-        nonlocal_[i].zero();
-        for (int isym = 0; isym < sym.size(); isym++) {
-            int pr            = sym[isym].spg_op.proper;
-            auto eang         = sym[isym].spg_op.euler_angles;
-            auto rotm         = sht::rotation_matrix<double>(4, eang, pr);
-            auto spin_rot_su2 = rotation_matrix_su2(sym[isym].spin_rotation);
-
-            int iap = sym[isym].spg_op.inv_sym_atom[ia];
-            int jap = sym[isym].spg_op.inv_sym_atom[ja];
-
-            auto Ttot = sym[isym].spg_op.inv_sym_atom_T[ja] - sym[isym].spg_op.inv_sym_atom_T[ia] +
-                        dot(sym[isym].spg_op.invR, r3::vector<int>(T));
-
-            /* we must search for the right hubbard subspace since we may have
-             * multiple orbitals involved in the hubbard correction */
-
-            /* NOTE : the atom order is important here. */
-            int at1_lvl    = find_orbital_index(iap, n1, il);
-            int at2_lvl    = find_orbital_index(jap, n2, jl);
-            auto& occ_mtrx = occ_mtrx_T_[Ttot];
-
-            sddk::mdarray<std::complex<double>, 3> dm_ia_ja(2 * il + 1, 2 * jl + 1, ctx_.num_spins());
-            dm_ia_ja.zero();
-            /* apply spatial rotation */
-            for (int ispn = 0; ispn < ctx_.num_spins(); ispn++) {
-                for (int m1 = 0; m1 < ib; m1++) {
-                    for (int m2 = 0; m2 < jb; m2++) {
-                        for (int m1p = 0; m1p < ib; m1p++) {
-                            for (int m2p = 0; m2p < jb; m2p++) {
-                                dm_ia_ja(m1, m2, ispn) +=
-                                    rotm[il](m1, m1p) * rotm[jl](m2, m2p) *
-                                    occ_mtrx(offset_[at1_lvl] + m1p, offset_[at2_lvl] + m2p, ispn) * f;
-                            }
-                        }
-                    }
-                }
-            }
-
-            if (ctx_.num_mag_dims() == 0) {
-                for (int m1 = 0; m1 < ib; m1++) {
-                    for (int m2 = 0; m2 < jb; m2++) {
-                        nonlocal_[i](m1, m2, 0) += dm_ia_ja(m1, m2, 0);
-                    }
-                }
-            }
-            if (ctx_.num_mag_dims() == 1) {
-                int const map_s[3][2] = {{0, 0}, {1, 1}, {0, 1}};
-                for (int j = 0; j < 2; j++) {
-                    int s1 = map_s[j][0];
-                    int s2 = map_s[j][1];
-
-                    for (int m1 = 0; m1 < ib; m1++) {
-                        for (int m2 = 0; m2 < jb; m2++) {
-                            std::complex<double> dm[2][2] = {{dm_ia_ja(m1, m2, 0), 0}, {0, dm_ia_ja(m1, m2, 1)}};
-
-                            for (int s1p = 0; s1p < 2; s1p++) {
-                                for (int s2p = 0; s2p < 2; s2p++) {
-                                    nonlocal_[i](m1, m2, j) +=
-                                        dm[s1p][s2p] * spin_rot_su2(s1, s1p) * std::conj(spin_rot_su2(s2, s2p));
-                                }
-                            }
-                        }
-                    }
-                }
-            }
-        }
-    }
-}
 void
 Occupation_matrix::init()
 {
@@ -444,20 +235,19 @@ Occupation_matrix::init()
     this->zero();
     #pragma omp parallel for schedule(static)
     for (int at_lvl = 0; at_lvl < static_cast<int>(local_.size()); at_lvl++) {
-        const int ia     = atomic_orbitals_[at_lvl].first;
-        auto const& atom = ctx_.unit_cell().atom(ia);
+        const int ia      = atomic_orbitals_[at_lvl].first;
+        auto const& atom  = ctx_.unit_cell().atom(ia);
+        const int il      = atom.type().lo_descriptor_hub(atomic_orbitals_[at_lvl].second).l();
+        const int lmax_at = 2 * il + 1;
 
         if (atom.type().lo_descriptor_hub(atomic_orbitals_[at_lvl].second).use_for_calculation()) {
-
-            int il            = atom.type().lo_descriptor_hub(atomic_orbitals_[at_lvl].second).l();
-            const int lmax_at = 2 * il + 1;
             if (atom.type().lo_descriptor_hub(atomic_orbitals_[at_lvl].second).initial_occupancy().size()) {
-              /* if we specify the occcupancy in the input file */
-              for (int ispn = 0; ispn < ctx_.num_spins(); ispn++) {
+                /* if we specify the occcupancy in the input file */
+                for (int ispn = 0; ispn < ctx_.num_spins(); ispn++) {
                     for (int m = 0; m < lmax_at; m++) {
                         this->local_[at_lvl](m, m, ispn) = atom.type()
-                                                               .lo_descriptor_hub(atomic_orbitals_[at_lvl].second)
-                                                               .initial_occupancy()[m + ispn * lmax_at];
+                                                                   .lo_descriptor_hub(atomic_orbitals_[at_lvl].second)
+                                                                   .initial_occupancy()[m + ispn * lmax_at];
                     }
                 }
             } else {
@@ -484,7 +274,7 @@ Occupation_matrix::init()
                             for (int m = 0; m < lmax_at; m++) {
                                 this->local_[at_lvl](m, m, majs) = 1.0;
                                 this->local_[at_lvl](m, m, mins) =
-                                    (charge - static_cast<double>(lmax_at)) / static_cast<double>(lmax_at);
+                                        (charge - static_cast<double>(lmax_at)) / static_cast<double>(lmax_at);
                             }
                         } else {
                             for (int m = 0; m < lmax_at; m++) {
@@ -494,9 +284,9 @@ Occupation_matrix::init()
                     } else {
                         // double c1, s1;
                         // sincos(atom.type().starting_magnetization_theta(), &s1, &c1);
-                        double c1 = atom.vector_field()[2];
-                        std::complex<double> cs =
-                            std::complex<double>(atom.vector_field()[0], atom.vector_field()[1]) / sqrt(1.0 - c1 * c1);
+                        double c1               = atom.vector_field()[2];
+                        std::complex<double> cs = std::complex<double>(atom.vector_field()[0], atom.vector_field()[1]) /
+                                                  sqrt(1.0 - c1 * c1);
                         std::complex<double> ns[4];
 
                         if (charge > (lmax_at)) {
@@ -532,12 +322,46 @@ Occupation_matrix::init()
                     }
                 }
             }
+            // initialize the occupancy matrices to their user provided values.
+            if (ctx_.cfg().hubbard().constrained_calculation() && apply_constraints_.size()) {
+                if (apply_constraints_[at_lvl]) {
+                    copy(local_constraints_[at_lvl], local_[at_lvl]);
+                }
+            }
         }
     }
-
     print_occupancies(2);
 }
 
+void
+Occupation_matrix::calculate_constraints_and_error()
+{
+    if (apply_constraint()) {
+        double error_ = 0.0;
+        for (int at_lvl = 0; at_lvl < static_cast<int>(local_.size()); at_lvl++) {
+            if (apply_constraints_[at_lvl]) {
+                const int ia      = atomic_orbitals_[at_lvl].first;
+                auto const& atom  = ctx_.unit_cell().atom(ia);
+                int il            = atom.type().lo_descriptor_hub(atomic_orbitals_[at_lvl].second).l();
+                const int lmax_at = 2 * il + 1;
+                for (int is = 0; is < ctx_.num_spins(); is++) {
+                    for (int m1 = 0; m1 < lmax_at; m1++) {
+                        for (int m2 = 0; m2 < lmax_at; m2++) {
+                            std::complex<double> tmp =
+                                    this->local_[at_lvl](m2, m1, is) - this->local_constraints_[at_lvl](m2, m1, is);
+                            multipliers_constraints_[at_lvl](m2, m1, is) +=
+                                    tmp * ctx_.cfg().hubbard().constraint_beta_mixing();
+                            error_ = std::max(error_, std::abs(tmp));
+                        }
+                    }
+                }
+            }
+        }
+        this->constraint_error_ = error_;
+        this->num_steps_++;
+    }
+}
+
 void
 Occupation_matrix::print_occupancies(int verbosity__) const
 {
