@@ -24,7 +24,9 @@
 
 #include <iomanip>
 #include "dft_ground_state.hpp"
-#include "utils/profiler.hpp"
+#include "core/profiler.hpp"
+#include "hamiltonian/initialize_subspace.hpp"
+#include "hamiltonian/diagonalize.hpp"
 
 namespace sirius {
 
@@ -37,20 +39,28 @@ DFT_ground_state::initial_state()
     potential_.generate(density_, ctx_.use_symmetry(), true);
     if (!ctx_.full_potential()) {
         if (ctx_.cfg().parameters().precision_wf() == "fp32") {
-#if defined(USE_FP32)
+#if defined(SIRIUS_USE_FP32)
             Hamiltonian0<float> H0(potential_, true);
-            Band(ctx_).initialize_subspace(kset_, H0);
+            initialize_subspace(kset_, H0);
 #else
             RTE_THROW("not compiled with FP32 support");
 #endif
 
         } else {
             Hamiltonian0<double> H0(potential_, true);
-            Band(ctx_).initialize_subspace(kset_, H0);
+            initialize_subspace(kset_, H0);
         }
     }
 }
 
+void
+DFT_ground_state::create_H0()
+{
+    PROFILE("sirius::DFT_ground_state::create_H0");
+
+    H0_ = std::shared_ptr<Hamiltonian0<double>>(new Hamiltonian0<double>(potential_, true));
+}
+
 void
 DFT_ground_state::update()
 {
@@ -71,13 +81,12 @@ DFT_ground_state::energy_kin_sum_pw() const
 {
     double ekin{0};
 
-    for (int ikloc = 0; ikloc < kset_.spl_num_kpoints().local_size(); ikloc++) {
-        int ik  = kset_.spl_num_kpoints(ikloc);
-        auto kp = kset_.get<double>(ik);
+    for (auto it : kset_.spl_num_kpoints()) {
+        auto kp = kset_.get<double>(it.i);
 
         #pragma omp parallel for schedule(static) reduction(+:ekin)
         for (int igloc = 0; igloc < kp->num_gkvec_loc(); igloc++) {
-            auto Gk = kp->gkvec().gkvec_cart<sddk::index_domain_t::local>(igloc);
+            auto Gk = kp->gkvec().gkvec_cart<index_domain_t::local>(igloc);
 
             double d{0};
             for (int ispin = 0; ispin < ctx_.num_spins(); ispin++) {
@@ -128,31 +137,31 @@ DFT_ground_state::check_scf_density()
     bool precompute_lapw{true};
     Hamiltonian0<double> H0(pot, precompute_lapw);
     /* initialize the subspace */
-    Band(ctx_).initialize_subspace(kset_, H0);
+    ::sirius::initialize_subspace(kset_, H0);
     /* find new wave-functions */
-    Band(ctx_).solve<double, double>(kset_, H0, ctx_.cfg().settings().itsol_tol_min());
+    ::sirius::diagonalize<double, double>(H0, kset_, ctx_.cfg().settings().itsol_tol_min());
     /* find band occupancies */
     kset_.find_band_occupancies<double>();
     /* generate new density from the occupied wave-functions */
-    bool symmetrize{true};
     bool add_core{true};
     /* create new density */
-    Density rho1(ctx_);
-    rho1.generate<double>(kset_, symmetrize, add_core, transform_to_rg);
+    Density rho(ctx_);
+    rho.generate<double>(kset_, ctx_.use_symmetry(), add_core, transform_to_rg);
 
-    auto gs1 = energy_dict(ctx_, kset_, rho1, pot, ewald_energy_);
+    auto gs1 = energy_dict(ctx_, kset_, rho, pot, ewald_energy_);
 
-    auto calc_rms = [&](Field4D& a, Field4D& b) -> double
-    {
+    auto calc_rms = [&](Field4D& a, Field4D& b) -> double {
         double rms{0};
-        for (int ig = 0; ig < ctx_.gvec().count(); ig++) {
-            rms += std::pow(std::abs(a.component(0).f_pw_local(ig) - b.component(0).f_pw_local(ig)), 2);
+        for (int j = 0; j < ctx_.num_mag_dims() + 1; j++) {
+            for (int ig = 0; ig < ctx_.gvec().count(); ig++) {
+                rms += std::pow(std::abs(a.component(j).rg().f_pw_local(ig) - b.component(j).rg().f_pw_local(ig)), 2);
+            }
         }
         ctx_.comm().allreduce(&rms, 1);
         return std::sqrt(rms / ctx_.gvec().num_gvec());
     };
 
-    double rms = calc_rms(density_, rho1);
+    double rms      = calc_rms(density_, rho);
     double rms_veff = calc_rms(potential_, pot);
 
     json dict;
@@ -168,11 +177,12 @@ DFT_ground_state::check_scf_density()
                             << " Enew: " << gs1["energy"]["total"].get<double>() << std::endl;
 
         std::vector<std::string> labels({"total", "vha", "vxc", "exc", "bxc", "veff", "eval_sum", "kin", "ewald",
-                "vloc", "scf_correction", "entropy_sum"});
+                                         "vloc", "scf_correction", "entropy_sum"});
 
-        for (auto e: labels) {
-            RTE_OUT(ctx_.out()) << "energy component: " << e << ", diff: " <<
-                std::abs(gs0["energy"][e].get<double>() - gs1["energy"][e].get<double>()) << std::endl;
+        for (auto e : labels) {
+            RTE_OUT(ctx_.out()) << "energy component: " << e << ", diff: "
+                                << std::abs(gs0["energy"][e].get<double>() - gs1["energy"][e].get<double>())
+                                << std::endl;
         }
     }
 
@@ -210,18 +220,21 @@ DFT_ground_state::find(double density_tol__, double energy_tol__, double iter_so
         std::stringstream s;
         s << std::endl;
         s << "+------------------------------+" << std::endl
-          << "| SCF iteration " << std::setw(3) << iter << " out of " << std::setw(3) << num_dft_iter__ << '|' << std::endl
+          << "| SCF iteration " << std::setw(3) << iter << " out of " << std::setw(3) << num_dft_iter__ << '|'
+          << std::endl
           << "+------------------------------+" << std::endl;
         ctx_.message(2, __func__, s);
 
+        diagonalize_result_t result;
+
         if (ctx_.cfg().parameters().precision_wf() == "fp32") {
-#if defined(USE_FP32)
+#if defined(SIRIUS_USE_FP32)
             Hamiltonian0<float> H0(potential_, true);
             /* find new wave-functions */
             if (ctx_.cfg().parameters().precision_hs() == "fp32") {
-                Band(ctx_).solve<float, float>(kset_, H0, iter_solver_tol__);
+                result = sirius::diagonalize<float, float>(H0, kset_, iter_solver_tol__);
             } else {
-                Band(ctx_).solve<float, double>(kset_, H0, iter_solver_tol__);
+                result = sirius::diagonalize<float, double>(H0, kset_, iter_solver_tol__);
             }
             /* find band occupancies */
             kset_.find_band_occupancies<float>();
@@ -233,7 +246,7 @@ DFT_ground_state::find(double density_tol__, double energy_tol__, double iter_so
         } else {
             Hamiltonian0<double> H0(potential_, true);
             /* find new wave-functions */
-            Band(ctx_).solve<double, double>(kset_, H0, iter_solver_tol__);
+            result = sirius::diagonalize<double, double>(H0, kset_, iter_solver_tol__);
             /* find band occupancies */
             kset_.find_band_occupancies<double>();
             /* generate new density from the occupied wave-functions */
@@ -251,7 +264,7 @@ DFT_ground_state::find(double density_tol__, double energy_tol__, double iter_so
         /* estimate new tolerance of the iterative solver */
         double tol = rms;
         if (ctx_.cfg().mixer().use_hartree()) {
-            //tol = rms * rms / std::max(1.0, unit_cell_.num_electrons());
+            // tol = rms * rms / std::max(1.0, unit_cell_.num_electrons());
             tol = eha_res / std::max(1.0, unit_cell_.num_electrons());
         }
         tol = std::min(ctx_.cfg().settings().itsol_tol_scale()[0] * tol,
@@ -259,12 +272,18 @@ DFT_ground_state::find(double density_tol__, double energy_tol__, double iter_so
         /* tolerance can't be too small */
         iter_solver_tol__ = std::max(ctx_.cfg().settings().itsol_tol_min(), tol);
 
-#if defined(USE_FP32)
+        bool iter_solver_converged{true};
+        if (ctx_.cfg().iterative_solver().type() != "exact") {
+            iter_solver_converged = (tol <= ctx_.cfg().settings().itsol_tol_min());
+        }
+
+#if defined(SIRIUS_USE_FP32)
         if (ctx_.cfg().parameters().precision_gs() != "auto") {
             /* if the final precision is not equal to the current precision */
             if (ctx_.cfg().parameters().precision_gs() == "fp64" && ctx_.cfg().parameters().precision_wf() == "fp32") {
                 /* if we reached the mimimum tolerance for fp32 */
-                if ((ctx_.cfg().settings().fp32_to_fp64_rms() == 0 && iter_solver_tol__ <= ctx_.cfg().settings().itsol_tol_min()) ||
+                if ((ctx_.cfg().settings().fp32_to_fp64_rms() == 0 &&
+                     iter_solver_tol__ <= ctx_.cfg().settings().itsol_tol_min()) ||
                     (rms < ctx_.cfg().settings().fp32_to_fp64_rms())) {
                     std::cout << "switching to FP64" << std::endl;
                     ctx_.cfg().unlock();
@@ -273,20 +292,20 @@ DFT_ground_state::find(double density_tol__, double energy_tol__, double iter_so
                     ctx_.cfg().parameters().precision_hs("fp64");
                     ctx_.cfg().lock();
 
-                    for (int ikloc = 0; ikloc < kset_.spl_num_kpoints().local_size(); ikloc++) {
-                        int ik = kset_.spl_num_kpoints(ikloc);
+                    for (auto it : kset_.spl_num_kpoints()) {
                         for (int ispn = 0; ispn < ctx_.num_spins(); ispn++) {
-                            wf::copy(sddk::memory_t::host, kset_.get<float>(ik)->spinor_wave_functions(),
-                                    wf::spin_index(ispn), wf::band_range(0, ctx_.num_bands()),
-                                    kset_.get<double>(ik)->spinor_wave_functions(), wf::spin_index(ispn),
-                                    wf::band_range(0, ctx_.num_bands()));
+                            wf::copy(memory_t::host, kset_.get<float>(it.i)->spinor_wave_functions(),
+                                     wf::spin_index(ispn), wf::band_range(0, ctx_.num_bands()),
+                                     kset_.get<double>(it.i)->spinor_wave_functions(), wf::spin_index(ispn),
+                                     wf::band_range(0, ctx_.num_bands()));
                         }
                     }
                     for (int ik = 0; ik < kset_.num_kpoints(); ik++) {
                         for (int ispn = 0; ispn < ctx_.num_spinors(); ispn++) {
                             for (int j = 0; j < ctx_.num_bands(); j++) {
                                 kset_.get<double>(ik)->band_energy(j, ispn, kset_.get<float>(ik)->band_energy(j, ispn));
-                                kset_.get<double>(ik)->band_occupancy(j, ispn, kset_.get<float>(ik)->band_occupancy(j, ispn));
+                                kset_.get<double>(ik)->band_occupancy(j, ispn,
+                                                                      kset_.get<float>(ik)->band_occupancy(j, ispn));
                             }
                         }
                     }
@@ -310,7 +329,7 @@ DFT_ground_state::find(double density_tol__, double energy_tol__, double iter_so
         }
 
         if (ctx_.cfg().parameters().use_scf_correction()) {
-            double e2 = energy_potential(rho1, potential_);
+            double e2                    = energy_potential(rho1, potential_);
             this->scf_correction_energy_ = e2 - e1;
         }
 
@@ -327,12 +346,20 @@ DFT_ground_state::find(double density_tol__, double energy_tol__, double iter_so
         print_info(out);
         out << std::endl;
         out << "iteration : " << iter << ", RMS : " << std::setprecision(12) << std::scientific << rms
-            << ", energy difference : " << std::setprecision(12) << std::scientific << etot - eold << std::endl
-            << "residual density Hartree energy : " << eha_res;
+            << ", energy difference : " << std::setprecision(12) << std::scientific << etot - eold;
+        if (!ctx_.full_potential()) {
+            out << std::endl
+                << "Hartree energy of density residual : " << eha_res << std::endl
+                << "bands are converged : " << boolstr(result.converged);
+        }
+        if (ctx_.cfg().iterative_solver().type() != "exact") {
+            out << std::endl << "iterative solver converged : " << boolstr(iter_solver_converged);
+        }
+
         ctx_.message(2, __func__, out);
         /* check if the calculation has converged */
         bool converged{true};
-        converged = converged && (std::abs(eold - etot) < energy_tol__);
+        converged = (std::abs(eold - etot) < energy_tol__) && result.converged && iter_solver_converged;
         if (ctx_.cfg().mixer().use_hartree()) {
             converged = converged && (eha_res < density_tol__);
         } else {
@@ -341,7 +368,7 @@ DFT_ground_state::find(double density_tol__, double energy_tol__, double iter_so
         if (converged) {
             std::stringstream out;
             out << std::endl;
-            out << "converged after " << iter + 1 << " SCF iterations!";
+            out << "converged after " << iter + 1 << " SCF iterations!" << std::endl;
             ctx_.message(1, __func__, out);
             density_.check_num_electrons();
             num_iter = iter;
@@ -356,15 +383,15 @@ DFT_ground_state::find(double density_tol__, double energy_tol__, double iter_so
     ctx_.message(1, __func__, out);
 
     if (write_state__) {
-        ctx_.create_storage_file();
+        ctx_.create_storage_file(storage_file_name);
         if (ctx_.full_potential()) { // TODO: why this is necessary?
-            density_.rho().fft_transform(-1);
+            density_.rho().rg().fft_transform(-1);
             for (int j = 0; j < ctx_.num_mag_dims(); j++) {
-                density_.magnetization(j).fft_transform(-1);
+                density_.mag(j).rg().fft_transform(-1);
             }
         }
-        potential_.save();
-        density_.save();
+        potential_.save(storage_file_name);
+        density_.save(storage_file_name);
         // kset_.save(storage_file_name);
     }
 
@@ -374,10 +401,10 @@ DFT_ground_state::find(double density_tol__, double energy_tol__, double iter_so
 
     /* check density */
     if (num_iter >= 0) {
-        density_.rho().fft_transform(1);
+        density_.rho().rg().fft_transform(1);
         double rho_min{1e100};
-        for (int ir = 0; ir < density_.rho().spfft().local_slice_size(); ir++) {
-            rho_min = std::min(rho_min, density_.rho().f_rg(ir));
+        for (int ir = 0; ir < density_.rho().rg().spfft().local_slice_size(); ir++) {
+            rho_min = std::min(rho_min, density_.rho().rg().value(ir));
         }
         dict["rho_min"] = rho_min;
         ctx_.comm().allreduce<double, mpi::op_t::min>(&rho_min, 1);
@@ -393,9 +420,9 @@ DFT_ground_state::find(double density_tol__, double energy_tol__, double iter_so
         dict["converged"] = false;
     }
 
-    //if (ctx_.cfg().control().verification() >= 1) {
-    //    check_scf_density();
-    //}
+    if (env::check_scf_density()) {
+        check_scf_density();
+    }
 
     // dict["volume"] = ctx.unit_cell().omega() * std::pow(bohr_radius, 3);
     // dict["volume_units"] = "angstrom^3";
@@ -432,19 +459,17 @@ DFT_ground_state::print_info(std::ostream& out__) const
     density_.print_info(out__);
 
     out__ << std::endl;
-    out__ << "Energy" << std::endl
-          << utils::hbar(80, '-') << std::endl;
+    out__ << "Energy" << std::endl << hbar(80, '-') << std::endl;
 
     auto write_energy = [&](std::string label__, double value__) {
         out__ << std::left << std::setw(30) << label__ << " : " << std::right << std::setw(16) << std::setprecision(8)
               << std::fixed << value__ << std::endl;
     };
 
-    auto write_energy2 = [&](std::string label__, double value__)
-    {
-        out__ << std::left << std::setw(30) << label__ << " : " << std::right
-              << std::setw(16) << std::setprecision(8) << std::fixed << value__ << " (Ha), "
-              << std::setw(16) << std::setprecision(8) << std::fixed << value__ * 2 << " (Ry)" << std::endl;
+    auto write_energy2 = [&](std::string label__, double value__) {
+        out__ << std::left << std::setw(30) << label__ << " : " << std::right << std::setw(16) << std::setprecision(8)
+              << std::fixed << value__ << " (Ha), " << std::setw(16) << std::setprecision(8) << std::fixed
+              << value__ * 2 << " (Ry)" << std::endl;
     };
 
     write_energy("valence_eval_sum", evalsum1);
@@ -462,7 +487,7 @@ DFT_ground_state::print_info(std::ostream& out__) const
         write_energy("hartree contribution", 0.5 * evha);
         write_energy("xc contribution", eexc);
         write_energy("ewald contribution", ewald_energy_);
-        write_energy("PAW contribution", potential_.PAW_total_energy());
+        write_energy("PAW contribution", potential_.PAW_total_energy(density_));
     }
     write_energy("smearing (-TS)", s_sum);
     write_energy("SCF correction", this->scf_correction_energy_);
