@@ -26,11 +26,10 @@
 #define __DFT_GROUND_STATE_HPP__
 
 #include "k_point/k_point_set.hpp"
-#include "utils/json.hpp"
+#include "core/json.hpp"
 #include "hubbard/hubbard.hpp"
 #include "geometry/stress.hpp"
 #include "geometry/force.hpp"
-#include "band/band.hpp"
 #include "energy.hpp"
 
 using json = nlohmann::json;
@@ -66,6 +65,9 @@ class DFT_ground_state
     /// Atomic forces.
     Force forces_;
 
+    /// k-point independent part of the Hamiltonian.
+    std::shared_ptr<Hamiltonian0<double>> H0_; // hard-code double for now
+
     /// Store Ewald energy which is computed once and which doesn't change during the run.
     double ewald_energy_{0};
 
@@ -108,66 +110,91 @@ class DFT_ground_state
     }
 
     /// Return reference to a simulation context.
-    inline Simulation_context const& ctx() const
+    inline Simulation_context const&
+    ctx() const
     {
         return ctx_;
     }
 
-    inline Density& density()
+    inline Hamiltonian0<double>&
+    get_H0() const
+    {
+        return *H0_;
+    }
+
+    inline Density&
+    density()
     {
         return density_;
     }
 
-    inline Potential& potential()
+    inline Potential&
+    potential()
     {
         return potential_;
     }
 
-    inline K_point_set& k_point_set()
+    inline K_point_set&
+    k_point_set()
     {
         return kset_;
     }
 
-    inline Force& forces()
+    inline Force&
+    forces()
     {
         return forces_;
     }
 
-    inline Stress& stress()
+    inline Stress&
+    stress()
     {
         return stress_;
     }
 
-    inline double ewald_energy() const
+    inline double
+    ewald_energy() const
     {
         return ewald_energy_;
     }
 
-    inline double scf_correction_energy() const
+    inline double
+    scf_correction_energy() const
     {
         return scf_correction_energy_;
     }
 
-    double total_energy() const;
+    void
+    create_H0();
+
+    double
+    total_energy() const;
 
     /// Generate initial density, potential and a subspace of wave-functions.
-    void initial_state();
+    void
+    initial_state();
 
     /// Update the parameters after the change of lattice vectors or atomic positions.
-    void update();
+    void
+    update();
 
     /// Run the SCF ground state calculation and find a total energy minimum.
-    json find(double density_tol, double energy_tol, double initial_tolerance, int num_dft_iter, bool write_state);
+    json
+    find(double density_tol, double energy_tol, double initial_tolerance, int num_dft_iter, bool write_state);
 
     /// Print the basic information (total energy, charges, moments, etc.).
-    void print_info(std::ostream& out__) const;
+    void
+    print_info(std::ostream& out__) const;
 
-    double energy_kin_sum_pw() const;
+    double
+    energy_kin_sum_pw() const;
 
-    json serialize();
+    json
+    serialize();
 
     /// A quick check of self-constent density in case of pseudopotential.
-    json check_scf_density();
+    json
+    check_scf_density();
 };
 
 } // namespace sirius
