@@ -34,12 +34,12 @@ ewald_energy(const Simulation_context& ctx, const fft::Gvec& gvec, const Unit_ce
 
     #pragma omp parallel for reduction(+ : ewald_g)
     for (int igloc = gvec.skip_g0(); igloc < gvec.count(); igloc++) {
-        double g2 = std::pow(gvec.gvec_len<sddk::index_domain_t::local>(igloc), 2);
+        double g2 = std::pow(gvec.gvec_len<index_domain_t::local>(igloc), 2);
 
         std::complex<double> rho(0, 0);
 
         for (int ia = 0; ia < unit_cell.num_atoms(); ia++) {
-            rho += ctx.gvec_phase_factor(gvec.gvec<sddk::index_domain_t::local>(igloc), ia) *
+            rho += ctx.gvec_phase_factor(gvec.gvec<index_domain_t::local>(igloc), ia) *
                    static_cast<double>(unit_cell.atom(ia).zn());
         }
 
@@ -95,7 +95,7 @@ energy_bxc(const Density& density, const Potential& potential)
 {
     double ebxc{0};
     for (int j = 0; j < density.ctx().num_mag_dims(); j++) {
-        ebxc += sirius::inner(density.magnetization(j), potential.effective_magnetic_field(j));
+        ebxc += sirius::inner(density.mag(j), potential.effective_magnetic_field(j));
     }
     return ebxc;
 }
@@ -106,10 +106,9 @@ energy_enuc(Simulation_context const& ctx, Potential const& potential)
     auto& unit_cell = ctx.unit_cell();
     double enuc{0};
     if (ctx.full_potential()) {
-        for (int ialoc = 0; ialoc < unit_cell.spl_num_atoms().local_size(); ialoc++) {
-            int ia = unit_cell.spl_num_atoms(ialoc);
-            int zn = unit_cell.atom(ia).zn();
-            enuc -= 0.5 * zn * potential.vh_el(ia);
+        for (auto it : unit_cell.spl_num_atoms()) {
+            int zn = unit_cell.atom(it.i).zn();
+            enuc -= 0.5 * zn * potential.vh_el(it.i);
         }
         ctx.comm().allreduce(&enuc, 1);
     }
@@ -119,7 +118,7 @@ energy_enuc(Simulation_context const& ctx, Potential const& potential)
 double
 energy_vloc(Density const& density, Potential const& potential)
 {
-    return sirius::inner(potential.local_potential(), density.rho());
+    return sirius::inner(potential.local_potential(), density.rho().rg());
 }
 
 double
@@ -151,23 +150,24 @@ energy_kin(Simulation_context const& ctx, K_point_set const& kset, Density const
 }
 
 double
-total_energy(Simulation_context const& ctx, K_point_set const& kset, Density const& density, Potential const& potential,
-             double ewald_energy)
+ks_energy(Simulation_context const& ctx, const std::map<std::string, double>& energies)
 {
     double tot_en{0};
 
     switch (ctx.electronic_structure_method()) {
         case electronic_structure_method_t::full_potential_lapwlo: {
-            tot_en = (energy_kin(ctx, kset, density, potential) + energy_exc(density, potential) +
-                      0.5 * energy_vha(potential) + energy_enuc(ctx, potential));
+            tot_en = energies.at("ekin") + energies.at("exc") + 0.5 * energies.at("vha") + energies.at("enuc");
             break;
         }
 
         case electronic_structure_method_t::pseudopotential: {
-            tot_en = (kset.valence_eval_sum() - energy_vxc(density, potential) - energy_bxc(density, potential) -
-                      potential.PAW_one_elec_energy(density) - one_electron_energy_hubbard(density, potential)) -
-                     0.5 * energy_vha(potential) + energy_exc(density, potential) + potential.PAW_total_energy() +
-                     ewald_energy + kset.entropy_sum() + ::sirius::hubbard_energy(density);
+            tot_en = energies.at("valence_eval_sum") - energies.at("vxc") - energies.at("bxc") -
+                     energies.at("PAW_one_elec");
+            tot_en += -0.5 * energies.at("vha") + energies.at("exc") + energies.at("PAW_total_energy") +
+                      energies.at("ewald");
+            if (ctx.hubbard_correction()) {
+                tot_en += energies.at("hubbard_energy") - energies.at("hubbard_one_el_contribution");
+            }
             break;
         }
     }
@@ -175,11 +175,81 @@ total_energy(Simulation_context const& ctx, K_point_set const& kset, Density con
     return tot_en;
 }
 
+double
+ks_energy(Simulation_context const& ctx, K_point_set const& kset, Density const& density, Potential const& potential,
+          double ewald_energy)
+{
+    return ks_energy(ctx, total_energy_components(ctx, kset, density, potential, ewald_energy));
+}
+
+double
+total_energy(Simulation_context const& ctx, K_point_set const& kset, Density const& density, Potential const& potential,
+             double ewald_energy)
+{
+
+    double eks = ks_energy(ctx, kset, density, potential, ewald_energy);
+    double tot_en{0};
+
+    switch (ctx.electronic_structure_method()) {
+        case electronic_structure_method_t::full_potential_lapwlo: {
+            tot_en = eks;
+            break;
+        }
+
+        case electronic_structure_method_t::pseudopotential: {
+            tot_en = eks + kset.entropy_sum();
+            break;
+        }
+        default: {
+            RTE_THROW("invalid electronic_structure_method");
+        }
+    }
+
+    return tot_en;
+}
+
+std::map<std::string, double>
+total_energy_components(Simulation_context const& ctx, const K_point_set& kset, Density const& density,
+                        Potential const& potential, double ewald_energy)
+{
+    std::map<std::string, double> table;
+    switch (ctx.electronic_structure_method()) {
+        case electronic_structure_method_t::full_potential_lapwlo: {
+            table["ekin"] = energy_kin(ctx, kset, density, potential);
+            table["exc"]  = energy_exc(density, potential);
+            table["vha"]  = energy_vha(potential);
+            table["enuc"] = energy_enuc(ctx, potential);
+            break;
+        }
+
+        case electronic_structure_method_t::pseudopotential: {
+            table["valence_eval_sum"] = kset.valence_eval_sum();
+            table["vxc"]              = energy_vxc(density, potential);
+            table["bxc"]              = energy_bxc(density, potential);
+            table["PAW_one_elec"]     = potential.PAW_one_elec_energy(density);
+            table["vha"]              = energy_vha(potential);
+            table["exc"]              = energy_exc(density, potential);
+            table["ewald"]            = ewald_energy;
+            table["PAW_total_energy"] = potential.PAW_total_energy(density);
+            break;
+        }
+    }
+
+    if (ctx.hubbard_correction()) {
+        table["hubbard_one_el_contribution"] = one_electron_energy_hubbard(density, potential);
+        table["hubbard_energy"]              = ::sirius::hubbard_energy(density);
+    }
+
+    table["entropy"] = kset.entropy_sum();
+
+    return table;
+}
+
 double
 hubbard_energy(Density const& density)
 {
     if (density.ctx().hubbard_correction()) {
-        return ::sirius::energy(density.occupation_matrix());
+        return energy(density.occupation_matrix());
     } else {
         return 0.0;
     }
@@ -197,7 +267,7 @@ one_electron_energy_hubbard(Density const& density, Potential const& potential)
 {
     auto& ctx = density.ctx();
     if (ctx.hubbard_correction()) {
-        return ::sirius::one_electron_energy_hubbard(density.occupation_matrix(), potential.hubbard_potential());
+        return one_electron_energy_hubbard(density.occupation_matrix(), potential.hubbard_potential());
     }
     return 0.0;
 }
