@@ -28,20 +28,21 @@
 #include "function3d/periodic_function.hpp"
 #include "k_point/k_point.hpp"
 #include "density/augmentation_operator.hpp"
-#include "beta_projectors/beta_projectors.hpp"
+#include "beta_projectors/beta_projectors_base.hpp"
 
 namespace sirius {
 
 /** \tparam T  Precision type of the wave-functions */
-template<typename T, typename F>
-void add_k_point_contribution_nonlocal(Simulation_context& ctx__, Beta_projectors_base<T>& bp_base__,
-        K_point<T>& kp__, sddk::mdarray<real_type<F>, 2>& collect_res__)
+template <typename T, typename F>
+void
+add_k_point_contribution_nonlocal(Simulation_context& ctx__, Beta_projectors_base<T>& bp_base__, K_point<T>& kp__,
+                                  mdarray<real_type<F>, 2>& collect_res__)
 {
     PROFILE("sirius::add_k_point_contribution_nonlocal");
 
     auto& uc = ctx__.unit_cell();
 
-    if (uc.mt_lo_basis_size() == 0) {
+    if (uc.max_mt_basis_size() == 0) {
         return;
     }
 
@@ -49,26 +50,31 @@ void add_k_point_contribution_nonlocal(Simulation_context& ctx__, Beta_projector
 
     double main_two_factor{-2};
 
+    auto mt          = ctx__.processing_unit_memory_t();
+    auto bp_gen      = bp.make_generator(mt);
+    auto beta_coeffs = bp_gen.prepare();
+
+    auto bp_base_gen      = bp_base__.make_generator(mt);
+    auto beta_coeffs_base = bp_base_gen.prepare();
+
     for (int icnk = 0; icnk < bp_base__.num_chunks(); icnk++) {
 
-        bp.prepare();
         /* generate chunk for inner product of beta */
-        bp.generate(ctx__.processing_unit_memory_t(), icnk);
+        bp_gen.generate(beta_coeffs, icnk);
 
         /* store <beta|psi> for spin up and down */
-        sddk::matrix<F> beta_phi_chunks[2];
+        matrix<F> beta_phi_chunks[2];
 
         for (int ispn = 0; ispn < ctx__.num_spins(); ispn++) {
-            int nbnd = kp__.num_occupied_bands(ispn);
-            beta_phi_chunks[ispn] = bp.template inner<F>(ctx__.processing_unit_memory_t(), icnk,
-                    kp__.spinor_wave_functions(), wf::spin_index(ispn), wf::band_range(0, nbnd));
+            int nbnd              = kp__.num_occupied_bands(ispn);
+            beta_phi_chunks[ispn] = inner_prod_beta<F>(ctx__.spla_context(), mt, ctx__.host_memory_t(),
+                                                       is_device_memory(mt), beta_coeffs, kp__.spinor_wave_functions(),
+                                                       wf::spin_index(ispn), wf::band_range(0, nbnd));
         }
-        bp.dismiss();
 
-        bp_base__.prepare();
         for (int x = 0; x < bp_base__.num_comp(); x++) {
             /* generate chunk for inner product of beta gradient */
-            bp_base__.generate(ctx__.processing_unit_memory_t(), icnk, x);
+            bp_base_gen.generate(beta_coeffs_base, icnk, x);
 
             for (int ispn = 0; ispn < ctx__.num_spins(); ispn++) {
                 int spin_factor = (ispn == 0 ? 1 : -1);
@@ -76,19 +82,20 @@ void add_k_point_contribution_nonlocal(Simulation_context& ctx__, Beta_projector
                 int nbnd = kp__.num_occupied_bands(ispn);
 
                 /* inner product of beta gradient and WF */
-                auto bp_base_phi_chunk = bp_base__.template inner<F>(ctx__.processing_unit_memory_t(), icnk,
+                auto bp_base_phi_chunk = inner_prod_beta<F>(
+                        ctx__.spla_context(), mt, ctx__.host_memory_t(), is_device_memory(mt), beta_coeffs_base,
                         kp__.spinor_wave_functions(), wf::spin_index(ispn), wf::band_range(0, nbnd));
 
-                sddk::splindex<sddk::splindex_t::block> spl_nbnd(nbnd, kp__.comm().size(), kp__.comm().rank());
+                splindex_block<> spl_nbnd(nbnd, n_blocks(kp__.comm().size()), block_id(kp__.comm().rank()));
 
                 int nbnd_loc = spl_nbnd.local_size();
 
                 #pragma omp parallel for
-                for (int ia_chunk = 0; ia_chunk < bp_base__.chunk(icnk).num_atoms_; ia_chunk++) {
-                    int ia =   bp_base__.chunk(icnk).desc_(beta_desc_idx::ia, ia_chunk);
-                    int offs = bp_base__.chunk(icnk).desc_(beta_desc_idx::offset, ia_chunk);
-                    int nbf =  bp_base__.chunk(icnk).desc_(beta_desc_idx::nbf, ia_chunk);
-                    int iat = uc.atom(ia).type_id();
+                for (int ia_chunk = 0; ia_chunk < beta_coeffs_base.beta_chunk_->num_atoms_; ia_chunk++) {
+                    int ia   = beta_coeffs_base.beta_chunk_->desc_(beta_desc_idx::ia, ia_chunk);
+                    int offs = beta_coeffs_base.beta_chunk_->desc_(beta_desc_idx::offset, ia_chunk);
+                    int nbf  = beta_coeffs_base.beta_chunk_->desc_(beta_desc_idx::nbf, ia_chunk);
+                    int iat  = uc.atom(ia).type_id();
 
                     if (uc.atom(ia).type().spin_orbit_coupling()) {
                         RTE_THROW("stress and forces with SO coupling are not upported");
@@ -96,14 +103,16 @@ void add_k_point_contribution_nonlocal(Simulation_context& ctx__, Beta_projector
 
                     /* helper lambda to calculate for sum loop over bands for different beta_phi and dij combinations*/
                     auto for_bnd = [&](int ibf, int jbf, std::complex<real_type<F>> dij, real_type<F> qij,
-                                       sddk::matrix<F>& beta_phi_chunk) {
-                        /* gather everything = - 2  Re[ occ(k,n) weight(k) beta_phi*(i,n) [Dij - E(n)Qij] beta_base_phi(j,n) ]*/
+                                       matrix<F>& beta_phi_chunk) {
+                        /* gather everything = - 2  Re[ occ(k,n) weight(k) beta_phi*(i,n) [Dij - E(n)Qij]
+                         * beta_base_phi(j,n) ]*/
                         for (int ibnd_loc = 0; ibnd_loc < nbnd_loc; ibnd_loc++) {
-                            int ibnd = spl_nbnd[ibnd_loc];
+                            int ibnd = spl_nbnd.global_index(ibnd_loc);
 
                             auto d1 = main_two_factor * kp__.band_occupancy(ibnd, ispn) * kp__.weight();
                             auto z2 = dij - static_cast<real_type<F>>(kp__.band_energy(ibnd, ispn) * qij);
-                            auto z1 = z2 * std::conj(beta_phi_chunk(offs + jbf, ibnd)) * bp_base_phi_chunk(offs + ibf, ibnd);
+                            auto z1 = z2 * std::conj(beta_phi_chunk(offs + jbf, ibnd)) *
+                                      bp_base_phi_chunk(offs + ibf, ibnd);
 
                             auto scalar_part = static_cast<real_type<F>>(d1) * z1;
 
@@ -113,10 +122,10 @@ void add_k_point_contribution_nonlocal(Simulation_context& ctx__, Beta_projector
                     };
 
                     for (int ibf = 0; ibf < nbf; ibf++) {
-                        int lm2 = uc.atom(ia).type().indexb(ibf).lm;
+                        int lm2    = uc.atom(ia).type().indexb(ibf).lm;
                         int idxrf2 = uc.atom(ia).type().indexb(ibf).idxrf;
                         for (int jbf = 0; jbf < nbf; jbf++) {
-                            int lm1 = uc.atom(ia).type().indexb(jbf).lm;
+                            int lm1    = uc.atom(ia).type().indexb(jbf).lm;
                             int idxrf1 = uc.atom(ia).type().indexb(jbf).idxrf;
 
                             /* Qij exists only in the case of ultrasoft/PAW */
@@ -159,20 +168,20 @@ void add_k_point_contribution_nonlocal(Simulation_context& ctx__, Beta_projector
                             if (ctx__.num_mag_dims() == 3) {
                                 /* Dij(10) = dij_X + i dij_Y ; Dij(01) = dij_X - i dij_Y */
                                 dij = std::complex<real_type<F>>(uc.atom(ia).d_mtrx(ibf, jbf, 2),
-                                                     spin_factor * uc.atom(ia).d_mtrx(ibf, jbf, 3));
+                                                                 spin_factor * uc.atom(ia).d_mtrx(ibf, jbf, 3));
                                 /* add non-diagonal spin components*/
                                 for_bnd(ibf, jbf, dij, 0.0, beta_phi_chunks[ispn + spin_factor]);
                             }
                         } // jbf
-                    } // ibf
-                } // ia_chunk
-            } // ispn
-        } // x
+                    }     // ibf
+                }         // ia_chunk
+            }             // ispn
+        }                 // x
     }
 
-    bp_base__.dismiss();
+    // bp_base__.dismiss();
 }
 
-}
+} // namespace sirius
 
 #endif /* __NON_LOCAL_FUNCTOR_HPP__ */
