@@ -46,8 +46,14 @@ Hubbard_matrix::Hubbard_matrix(Simulation_context& ctx__)
             }
         }
 
-        local_ = std::vector<sddk::mdarray<std::complex<double>, 3>>(num_atomic_level);
+        local_ = std::vector<mdarray<std::complex<double>, 3>>(num_atomic_level);
 
+        if (ctx_.cfg().hubbard().constrained_calculation() && ctx_.cfg().hubbard().local_constraint().size()) {
+            apply_constraints_.clear();
+            apply_constraints_.resize(num_atomic_level, false);
+            local_constraints_       = std::vector<mdarray<std::complex<double>, 3>>(num_atomic_level);
+            multipliers_constraints_ = std::vector<mdarray<std::complex<double>, 3>>(num_atomic_level);
+        }
         /* the offsets here match the offsets of the hubbard wave functions but
          * are more fine grained. The offsets of the hubbard wave functions are
          * for atom while here they are for each atomic level. Since all atomic
@@ -59,26 +65,75 @@ Hubbard_matrix::Hubbard_matrix(Simulation_context& ctx__)
 
         int size{0};
         for (int at_lvl = 0; at_lvl < static_cast<int>(local_.size()); at_lvl++) {
-            offset_[at_lvl] = size;
-            int ia          = atomic_orbitals_[at_lvl].first;
-            auto& atom_type = ctx_.unit_cell().atom(ia).type();
-            int lo_ind      = atomic_orbitals_[at_lvl].second;
-            int l           = atom_type.lo_descriptor_hub(lo_ind).l();
-            int mmax        = 2 * l + 1;
+            offset_[at_lvl]  = size;
+            const int ia     = atomic_orbitals_[at_lvl].first;
+            auto& atom_type  = ctx_.unit_cell().atom(ia).type();
+            const int lo_ind = atomic_orbitals_[at_lvl].second;
+            const int l      = atom_type.lo_descriptor_hub(lo_ind).l();
+            const int n      = atom_type.lo_descriptor_hub(lo_ind).n();
+            const int mmax   = 2 * l + 1;
 
-            local_[at_lvl] = sddk::mdarray<std::complex<double>, 3>(mmax, mmax, 4, sddk::memory_t::host, "local_hubbard");
+            local_[at_lvl] = mdarray<std::complex<double>, 3>({mmax, mmax, 4}, mdarray_label("local_hubbard"));
             local_[at_lvl].zero();
+
+            if (ctx_.cfg().hubbard().constrained_calculation()) {
+                local_constraints_[at_lvl] =
+                        mdarray<std::complex<double>, 3>({mmax, mmax, 4}, mdarray_label("local_hubbard_constraint"));
+                multipliers_constraints_[at_lvl] = mdarray<std::complex<double>, 3>(
+                        {mmax, mmax, 4}, mdarray_label("lagrange_multiplier_constraint"));
+                multipliers_constraints_[at_lvl].zero();
+                local_constraints_[at_lvl].zero();
+
+                // fill the constrained occupation numbers. I need to go through the full list of orbitals
+                for (int cts_index = 0; cts_index < ctx_.cfg().hubbard().local_constraint().size(); cts_index++) {
+                    const auto& constraint_    = ctx_.cfg().hubbard().local_constraint(cts_index);
+                    apply_constraints_[at_lvl] = (constraint_.atom_index() == ia) && (constraint_.l() == l) &&
+                                                 ((constraint_.n() == n) || (n < 0) || (constraint_.n() < 0));
+                    if (apply_constraints_[at_lvl]) {
+                        // fill the constrained occupation numbers. I need to go through the full for each orbital
+                        const auto& cts_ = ctx_.cfg().hubbard().local_constraint();
+                        for (int cts_index = 0; cts_index < cts_.size(); cts_index++) {
+                            const auto& constraint_ = ctx_.cfg().hubbard().local_constraint(cts_index);
+                            if ((constraint_.atom_index() == ia) && (constraint_.l() == l) &&
+                                (((n == constraint_.n()) || (n < 0)) || (constraint_.n() < 0))) {
+                                const auto& occ_matrix__ = constraint_.occupancy();
+                                if (constraint_.contains("lm_order")) {
+                                    const auto& lm_order_ = constraint_.lm_order();
+                                    for (unsigned int sp = 0; sp < occ_matrix__.size();
+                                         sp++) { // spin blocks up-up, up-down, down-down
+                                        for (int m1 = 0; m1 < mmax; m1++) {
+                                            for (int m2 = 0; m2 < mmax; m2++) {
+                                                local_constraints_[at_lvl](m2, m1, sp) =
+                                                        occ_matrix__[sp][l + lm_order_[m1]][l + lm_order_[m2]];
+                                            }
+                                        }
+                                    }
+                                } else {
+                                    for (unsigned int sp = 0; sp < occ_matrix__.size();
+                                         sp++) { // spin blocks up-up, up-down, down-down
+                                        for (int m1 = 0; m1 < mmax; m1++) {
+                                            for (int m2 = 0; m2 < mmax; m2++) {
+                                                local_constraints_[at_lvl](m2, m1, sp) = occ_matrix__[sp][m1][m2];
+                                            }
+                                        }
+                                    }
+                                }
+                            }
+                        }
+                    }
+                }
+            }
             size += mmax;
         }
 
         nonlocal_.clear();
-        nonlocal_ = std::vector<sddk::mdarray<std::complex<double>, 3>>(ctx_.cfg().hubbard().nonlocal().size());
+        nonlocal_ = std::vector<mdarray<std::complex<double>, 3>>(ctx_.cfg().hubbard().nonlocal().size());
         for (int i = 0; i < static_cast<int>(ctx_.cfg().hubbard().nonlocal().size()); i++) {
             auto nl      = ctx_.cfg().hubbard().nonlocal(i);
             int il       = nl.l()[0];
             int jl       = nl.l()[1];
-            nonlocal_[i] = sddk::mdarray<std::complex<double>, 3>(2 * il + 1, 2 * jl + 1, 4, sddk::memory_t::host,
-                    "nonlocal_hubbard");
+            nonlocal_[i] = mdarray<std::complex<double>, 3>({2 * il + 1, 2 * jl + 1, ctx_.num_spins()},
+                                                            mdarray_label("nonlocal_hubbard"));
             nonlocal_[i].zero();
         }
     }
@@ -93,12 +148,13 @@ Hubbard_matrix::access(std::string const& what__, std::complex<double>* occ__, i
         RTE_THROW(s);
     }
 
-    sddk::mdarray<std::complex<double>, 4> occ_mtrx;
+    mdarray<std::complex<double>, 4> occ_mtrx;
     /* in non-collinear case the occupancy matrix is complex */
     if (ctx_.num_mag_dims() == 3) {
-        occ_mtrx = sddk::mdarray<std::complex<double>, 4>(occ__, ld__, ld__, 4, ctx_.unit_cell().num_atoms());
+        occ_mtrx = mdarray<std::complex<double>, 4>({ld__, ld__, 4, ctx_.unit_cell().num_atoms()}, occ__);
     } else {
-        occ_mtrx = sddk::mdarray<std::complex<double>, 4>(occ__, ld__, ld__, ctx_.num_spins(), ctx_.unit_cell().num_atoms());
+        occ_mtrx =
+                mdarray<std::complex<double>, 4>({ld__, ld__, ctx_.num_spins(), ctx_.unit_cell().num_atoms()}, occ__);
     }
     if (what__ == "get") {
         occ_mtrx.zero();
@@ -143,7 +199,7 @@ Hubbard_matrix::print_local(int at_lvl__, std::ostream& out__) const
     if (ctx_.num_mag_dims() != 3) {
         int mmax = 2 * l + 1;
         for (int is = 0; is < ctx_.num_spins(); is++) {
-            out__ << utils::hbar(width * mmax, '-') << std::endl;
+            out__ << hbar(width * mmax, '-') << std::endl;
             bool has_imag{false};
             for (int m = 0; m < mmax; m++) {
                 for (int mp = 0; mp < mmax; mp++) {
@@ -164,10 +220,10 @@ Hubbard_matrix::print_local(int at_lvl__, std::ostream& out__) const
                 }
             }
         }
-        out__ << utils::hbar(width * mmax, '-') << std::endl;
+        out__ << hbar(width * mmax, '-') << std::endl;
     } else {
         int mmax = 2 * l + 1;
-        out__ << utils::hbar(2 * width * mmax + 3, '-') << std::endl;
+        out__ << hbar(2 * width * mmax + 3, '-') << std::endl;
         for (int m = 0; m < mmax; m++) {
             for (int mp = 0; mp < mmax; mp++) {
                 print_number(std::real(this->local(at_lvl__)(m, mp, 0)));
@@ -178,7 +234,7 @@ Hubbard_matrix::print_local(int at_lvl__, std::ostream& out__) const
             }
             out__ << std::endl;
         }
-        out__ << utils::hbar(2 * width * mmax + 3, '-') << std::endl;
+        out__ << hbar(2 * width * mmax + 3, '-') << std::endl;
         for (int m = 0; m < mmax; m++) {
             for (int mp = 0; mp < mmax; mp++) {
                 print_number(std::real(this->local(at_lvl__)(m, mp, 3)));
@@ -189,7 +245,11 @@ Hubbard_matrix::print_local(int at_lvl__, std::ostream& out__) const
             }
             out__ << std::endl;
         }
-        out__ << utils::hbar(2 * width * mmax + 3, '-') << std::endl;
+        out__ << hbar(2 * width * mmax + 3, '-') << std::endl;
+    }
+
+    if (apply_constraint()) {
+        out__ << "Hubbard constraint error (l2-norm): " << constraint_error_ << std::endl;
     }
 }
 
@@ -210,8 +270,8 @@ Hubbard_matrix::print_nonlocal(int idx__, std::ostream& out__) const
     /* convert to Cartesian coordinates */
     auto rc = dot(ctx_.unit_cell().lattice_vectors(), r);
 
-    out__ << "atom: " << ia << ", l: " << il << " -> atom: " << ja << ", l: " << jl << ", T: " << T
-          << ", r: " << rc << std::endl;
+    out__ << "atom: " << ia << ", l: " << il << " -> atom: " << ja << ", l: " << jl << ", T: " << T << ", r: " << rc
+          << std::endl;
 
     int const prec{5};
     int const width{10};
@@ -219,7 +279,7 @@ Hubbard_matrix::print_nonlocal(int idx__, std::ostream& out__) const
 
     if (ctx_.num_mag_dims() != 3) {
         for (int is = 0; is < ctx_.num_spins(); is++) {
-            out__ << utils::hbar(width * jb, '-') << std::endl;
+            out__ << hbar(width * jb, '-') << std::endl;
             bool has_imag{false};
             for (int m = 0; m < ib; m++) {
                 for (int mp = 0; mp < jb; mp++) {
@@ -240,7 +300,7 @@ Hubbard_matrix::print_nonlocal(int idx__, std::ostream& out__) const
                 }
             }
         }
-        out__ << utils::hbar(width * jb, '-') << std::endl;
+        out__ << hbar(width * jb, '-') << std::endl;
     }
     //} else {
     //    int l = atom.type().indexr_hub().am(0).l();
@@ -281,6 +341,12 @@ Hubbard_matrix::zero()
     for (int i = 0; i < static_cast<int>(ctx_.cfg().hubbard().nonlocal().size()); i++) {
         nonlocal_[i].zero();
     }
+
+    for (int at_lvl = 0; at_lvl < static_cast<int>(local_constraints_.size()); at_lvl++) {
+        if (apply_constraints(at_lvl)) {
+            multipliers_constraints_[at_lvl].zero();
+        }
+    }
 }
 
 } // namespace sirius
