@@ -27,32 +27,32 @@
 namespace sirius {
 
 template <typename T, sync_band_t what>
-void K_point_set::sync_band()
+void
+K_point_set::sync_band()
 {
     PROFILE("sirius::K_point_set::sync_band");
 
-    sddk::mdarray<double, 3> data(ctx_.num_bands(), ctx_.num_spinors(), num_kpoints(),
-            get_memory_pool(sddk::memory_t::host), "K_point_set::sync_band.data");
+    mdarray<double, 3> data({ctx_.num_bands(), ctx_.num_spinors(), num_kpoints()}, get_memory_pool(memory_t::host),
+                            mdarray_label("K_point_set::sync_band.data"));
+    data.zero();
 
     int nb = ctx_.num_bands() * ctx_.num_spinors();
     #pragma omp parallel
-    for (int ikloc = 0; ikloc < spl_num_kpoints_.local_size(); ikloc++) {
-        int ik = spl_num_kpoints_[ikloc];
-        auto kp = this->get<T>(ik);
+    for (auto it : spl_num_kpoints_) {
+        auto kp = this->get<T>(it.i);
         switch (what) {
             case sync_band_t::energy: {
-                std::copy(&kp->band_energies_(0, 0), &kp->band_energies_(0, 0) + nb, &data(0, 0, ik));
+                std::copy(&kp->band_energies_(0, 0), &kp->band_energies_(0, 0) + nb, &data(0, 0, it.i));
                 break;
             }
             case sync_band_t::occupancy: {
-                std::copy(&kp->band_occupancies_(0, 0), &kp->band_occupancies_(0, 0) + nb, &data(0, 0, ik));
+                std::copy(&kp->band_occupancies_(0, 0), &kp->band_occupancies_(0, 0) + nb, &data(0, 0, it.i));
                 break;
             }
         }
     }
 
-    comm().allgather(data.at(sddk::memory_t::host), nb * spl_num_kpoints_.local_size(),
-        nb * spl_num_kpoints_.global_offset());
+    comm().allreduce(data.at(memory_t::host), static_cast<int>(data.size()));
 
     #pragma omp parallel for
     for (int ik = 0; ik < num_kpoints(); ik++) {
@@ -70,37 +70,34 @@ void K_point_set::sync_band()
     }
 }
 
-template
-void
+template void
 K_point_set::sync_band<double, sync_band_t::energy>();
 
-template
-void
+template void
 K_point_set::sync_band<double, sync_band_t::occupancy>();
 
-#if defined(USE_FP32)
-template
-void
+#if defined(SIRIUS_USE_FP32)
+template void
 K_point_set::sync_band<float, sync_band_t::energy>();
 
-template
-void
+template void
 K_point_set::sync_band<float, sync_band_t::occupancy>();
 #endif
 
-void K_point_set::create_k_mesh(r3::vector<int> k_grid__, r3::vector<int> k_shift__, int use_symmetry__)
+void
+K_point_set::create_k_mesh(r3::vector<int> k_grid__, r3::vector<int> k_shift__, int use_symmetry__)
 {
     PROFILE("sirius::K_point_set::create_k_mesh");
 
     int nk;
-    sddk::mdarray<double, 2> kp;
+    mdarray<double, 2> kp;
     std::vector<double> wk;
     if (use_symmetry__) {
         auto result = get_irreducible_reciprocal_mesh(ctx_.unit_cell().symmetry(), k_grid__, k_shift__);
         nk          = std::get<0>(result);
         wk          = std::get<1>(result);
         auto tmp    = std::get<2>(result);
-        kp          = sddk::mdarray<double, 2>(3, nk);
+        kp          = mdarray<double, 2>({3, nk});
         for (int i = 0; i < nk; i++) {
             for (int x : {0, 1, 2}) {
                 kp(x, i) = tmp[i][x];
@@ -109,7 +106,7 @@ void K_point_set::create_k_mesh(r3::vector<int> k_grid__, r3::vector<int> k_shif
     } else {
         nk = k_grid__[0] * k_grid__[1] * k_grid__[2];
         wk = std::vector<double>(nk, 1.0 / nk);
-        kp = sddk::mdarray<double, 2>(3, nk);
+        kp = mdarray<double, 2>({3, nk});
 
         int ik = 0;
         for (int i0 = 0; i0 < k_grid__[0]; i0++) {
@@ -131,7 +128,8 @@ void K_point_set::create_k_mesh(r3::vector<int> k_grid__, r3::vector<int> k_shif
     initialize();
 }
 
-void K_point_set::initialize(std::vector<int> const& counts)
+void
+K_point_set::initialize(std::vector<int> const& counts)
 {
     if (this->initialized_) {
         RTE_THROW("K-point set is already initialized");
@@ -139,16 +137,18 @@ void K_point_set::initialize(std::vector<int> const& counts)
     PROFILE("sirius::K_point_set::initialize");
     /* distribute k-points along the 1-st dimension of the MPI grid */
     if (counts.empty()) {
-        sddk::splindex<sddk::splindex_t::block> spl_tmp(num_kpoints(), comm().size(), comm().rank());
-        spl_num_kpoints_ = sddk::splindex<sddk::splindex_t::chunk>(num_kpoints(), comm().size(), comm().rank(), spl_tmp.counts());
+        splindex_block<> spl_tmp(num_kpoints(), n_blocks(comm().size()), block_id(comm().rank()));
+        spl_num_kpoints_ = splindex_chunk<kp_index_t>(num_kpoints(), n_blocks(comm().size()), block_id(comm().rank()),
+                                                      spl_tmp.counts());
     } else {
-        spl_num_kpoints_ = sddk::splindex<sddk::splindex_t::chunk>(num_kpoints(), comm().size(), comm().rank(), counts);
+        spl_num_kpoints_ =
+                splindex_chunk<kp_index_t>(num_kpoints(), n_blocks(comm().size()), block_id(comm().rank()), counts);
     }
 
-    for (int ikloc = 0; ikloc < spl_num_kpoints_.local_size(); ikloc++) {
-        kpoints_[spl_num_kpoints_[ikloc]]->initialize();
-#if defined(USE_FP32)
-        kpoints_float_[spl_num_kpoints_[ikloc]]->initialize();
+    for (auto it : spl_num_kpoints_) {
+        kpoints_[it.i]->initialize();
+#if defined(SIRIUS_USE_FP32)
+        kpoints_float_[it.i]->initialize();
 #endif
     }
 
@@ -159,11 +159,11 @@ void K_point_set::initialize(std::vector<int> const& counts)
     this->initialized_ = true;
 }
 
-
-template<class F>
-double bisection_search(F&& f, double a, double b, double tol, int maxstep=1000)
+template <class F>
+double
+bisection_search(F&& f, double a, double b, double tol, int maxstep = 1000)
 {
-    double x = (a+b)/2;
+    double x  = (a + b) / 2;
     double fi = f(x);
     int step{0};
     /* compute fermy energy */
@@ -176,13 +176,13 @@ double bisection_search(F&& f, double a, double b, double tol, int maxstep=1000)
             a = x;
         }
 
-        x = (a + b) / 2.0;
+        x  = (a + b) / 2.0;
         fi = f(x);
 
         if (step > maxstep) {
             std::stringstream s;
             s << "search of band occupancies failed after 10000 steps";
-            TERMINATE(s);
+            RTE_THROW(s);
         }
         step++;
     }
@@ -203,39 +203,63 @@ double bisection_search(F&& f, double a, double b, double tol, int maxstep=1000)
  */
 template <class Nt, class DNt, class D2Nt>
 auto
-newton_minimization_chemical_potential(Nt&& N, DNt&& dN, D2Nt&& ddN, double mu0, double ne, double tol, int maxstep = 1000)
+newton_minimization_chemical_potential(Nt&& N, DNt&& dN, D2Nt&& ddN, double mu0, double ne, double tol,
+                                       int maxstep = 1000)
 {
-    struct {
-        double mu; // chemical potential
-        int iter; // newton information
+    // Newton finds the minimum, not necessarily N(mu) == ne, tolerate up to `tol_ne` difference in number of electrons
+    // if |N(mu_0) -ne| > tol_ne an error is thrown.
+    const double tol_ne = 1e-2;
+
+    struct
+    {
+        double mu;              // chemical potential
+        int iter{0};            // newton information
         std::vector<double> ys; // newton history
     } res;
+
     double mu = mu0;
     double alpha{1.0}; // Newton damping
     int iter{0};
+
+    if (std::abs(N(mu) - ne) < tol) {
+        res.mu   = mu;
+        res.iter = iter;
+        res.ys   = {};
+        return res;
+    }
+
     while (true) {
         // compute
         double Nf   = N(mu);
         double dNf  = dN(mu);
         double ddNf = ddN(mu);
         /* minimize (N(mu) - ne)^2  */
-        //double F = (Nf - ne) * (Nf - ne);
-        double dF = 2 * (Nf - ne) * dNf;
-        double ddF = 2 * dNf * dNf + 2 * (Nf - ne) * ddNf;
-        mu = mu - alpha * dF / std::abs(ddF);
+        // double F = (Nf - ne) * (Nf - ne);
+        double dF   = 2 * (Nf - ne) * dNf;
+        double ddF  = 2 * dNf * dNf + 2 * (Nf - ne) * ddNf;
+        double step = alpha * dF / std::abs(ddF);
+        mu          = mu - step;
 
         res.ys.push_back(mu);
 
-        if (std::abs(dF) < tol) {
-            res.iter = iter;
-            res.mu = mu;
-            return res;
+        if (std::abs(ddF) < 1e-30) {
+            std::stringstream s;
+            s << "Newton minimization (Fermi energy) failed because 2nd derivative too close to zero!"
+              << std::setprecision(8) << std::abs(Nf - ne) << "\n";
+            RTE_THROW(s);
         }
 
-        if (std::abs(ddF) < 1e-10) {
-            std::stringstream s;
-            s << "Newton minimization (chemical potential) failed because 2nd derivative too close to zero!";
-            RTE_THROW(s);
+        if (std::abs(step) < tol || std::abs(Nf - ne) < tol) {
+            if (std::abs(Nf - ne) > tol_ne) {
+                std::stringstream s;
+                s << "Newton minimization (Fermi energy) got stuck in a local minimum. Fallback to bisection search."
+                  << "\n";
+                RTE_THROW(s);
+            }
+
+            res.iter = iter;
+            res.mu   = mu;
+            return res;
         }
 
         iter++;
@@ -251,7 +275,8 @@ newton_minimization_chemical_potential(Nt&& N, DNt&& dN, D2Nt&& ddN, double mu0,
 }
 
 template <typename T>
-void K_point_set::find_band_occupancies()
+void
+K_point_set::find_band_occupancies()
 {
     PROFILE("sirius::K_point_set::find_band_occupancies");
 
@@ -280,12 +305,11 @@ void K_point_set::find_band_occupancies()
                 }
             }
         }
-        for (int ikloc = 0; ikloc < spl_num_kpoints_.local_size(); ikloc++) {
-            int ik = spl_num_kpoints_[ikloc];
+        for (auto it : spl_num_kpoints_) {
             for (int ispn = 0; ispn < ctx_.num_spinors(); ispn++) {
                 #pragma omp parallel for
                 for (int j = 0; j < ctx_.num_bands(); j++) {
-                    this->get<T>(ik)->band_occupancy(j, ispn, ctx_.max_occupancy());
+                    this->get<T>(it.i)->band_occupancy(j, ispn, ctx_.max_occupancy());
                 }
             }
         }
@@ -304,32 +328,30 @@ void K_point_set::find_band_occupancies()
     auto emax = std::numeric_limits<double>::lowest();
 
     #pragma omp parallel for reduction(min:emin) reduction(max:emax)
-    for (int ikloc = 0; ikloc < spl_num_kpoints_.local_size(); ikloc++) {
-        int ik = spl_num_kpoints_[ikloc];
+    for (auto it : spl_num_kpoints_) {
         for (int ispn = 0; ispn < ctx_.num_spinors(); ispn++) {
-            emin = std::min(emin, this->get<T>(ik)->band_energy(0, ispn));
-            emax = std::max(emax, this->get<T>(ik)->band_energy(ctx_.num_bands() - 1, ispn));
+            emin = std::min(emin, this->get<T>(it.i)->band_energy(0, ispn));
+            emax = std::max(emax, this->get<T>(it.i)->band_energy(ctx_.num_bands() - 1, ispn));
         }
     }
     comm().allreduce<double, mpi::op_t::min>(&emin, 1);
     comm().allreduce<double, mpi::op_t::max>(&emax, 1);
 
-    sddk::splindex<sddk::splindex_t::block> splb(ctx_.num_bands(), ctx_.comm_band().size(), ctx_.comm_band().rank());
+    splindex_block<> splb(ctx_.num_bands(), n_blocks(ctx_.comm_band().size()), block_id(ctx_.comm_band().rank()));
 
     /* computes N(ef; f) = \sum_{i,k} f(ef - e_{k,i}) */
     auto compute_ne = [&](double ef, auto&& f) {
         double ne{0};
-        for (int ikloc = 0; ikloc < spl_num_kpoints_.local_size(); ikloc++) {
-            int ik = spl_num_kpoints_[ikloc];
+        for (auto it : spl_num_kpoints_) {
             double tmp{0};
             #pragma omp parallel reduction(+ : tmp)
             for (int ispn = 0; ispn < ctx_.num_spinors(); ispn++) {
                 #pragma omp for
                 for (int j = 0; j < splb.local_size(); j++) {
-                    tmp += f(ef - this->get<T>(ik)->band_energy(splb[j], ispn)) * ctx_.max_occupancy();
+                    tmp += f(ef - this->get<T>(it.i)->band_energy(splb.global_index(j), ispn)) * ctx_.max_occupancy();
                 }
             }
-            ne += tmp * kpoints_[ik]->weight();
+            ne += tmp * kpoints_[it.i]->weight();
         }
         ctx_.comm().allreduce(&ne, 1);
         return ne;
@@ -345,40 +367,39 @@ void K_point_set::find_band_occupancies()
     }
 
     try {
-        auto F = [&compute_ne, ne_target, &f](double x) { return compute_ne(x, f) - ne_target; };
+        auto F        = [&compute_ne, ne_target, &f](double x) { return compute_ne(x, f) - ne_target; };
         energy_fermi_ = bisection_search(F, emin, emax, 1e-11);
 
         /* for cold and Methfessel Paxton smearing start newton minimization  */
-        if (ctx_.smearing() == smearing::smearing_t::cold || ctx_.smearing() == smearing::smearing_t::methfessel_paxton) {
-            f        = smearing::occupancy(ctx_.smearing(), ctx_.smearing_width());
-            auto df  = smearing::occupancy_deriv(ctx_.smearing(), ctx_.smearing_width());
-            auto ddf = smearing::occupancy_deriv2(ctx_.smearing(), ctx_.smearing_width());
-            auto N   = [&](double mu) { return compute_ne(mu, f); };
-            auto dN  = [&](double mu) { return compute_ne(mu, df); };
-            auto ddN = [&](double mu) { return compute_ne(mu, ddf); };
-            auto res_newton =  newton_minimization_chemical_potential(N, dN, ddN, energy_fermi_, ne_target, tol, 300);
-            energy_fermi_ = res_newton.mu;
+        if (ctx_.smearing() == smearing::smearing_t::cold ||
+            ctx_.smearing() == smearing::smearing_t::methfessel_paxton) {
+            f               = smearing::occupancy(ctx_.smearing(), ctx_.smearing_width());
+            auto df         = smearing::delta(ctx_.smearing(), ctx_.smearing_width());
+            auto ddf        = smearing::dxdelta(ctx_.smearing(), ctx_.smearing_width());
+            auto N          = [&](double mu) { return compute_ne(mu, f); };
+            auto dN         = [&](double mu) { return compute_ne(mu, df); };
+            auto ddN        = [&](double mu) { return compute_ne(mu, ddf); };
+            auto res_newton = newton_minimization_chemical_potential(N, dN, ddN, energy_fermi_, ne_target, tol, 1000);
+            energy_fermi_   = res_newton.mu;
             if (ctx_.verbosity() >= 2) {
                 RTE_OUT(ctx_.out()) << "newton iteration converged after " << res_newton.iter << " steps\n";
             }
         }
-    } catch(std::exception const& e) {
+    } catch (std::exception const& e) {
         if (ctx_.verbosity() >= 2) {
-            RTE_OUT(ctx_.out()) << e.what() << std::endl
-                << "fallback to bisection search" << std::endl;
+            RTE_OUT(ctx_.out()) << e.what() << std::endl << "fallback to bisection search" << std::endl;
         }
         f             = smearing::occupancy(ctx_.smearing(), ctx_.smearing_width());
         auto F        = [&compute_ne, ne_target, &f](double x) { return compute_ne(x, f) - ne_target; };
         energy_fermi_ = bisection_search(F, emin, emax, tol);
     }
 
-    for (int ikloc = 0; ikloc < spl_num_kpoints_.local_size(); ikloc++) {
-        int ik = spl_num_kpoints_[ikloc];
+    for (auto it : spl_num_kpoints_) {
         for (int ispn = 0; ispn < ctx_.num_spinors(); ispn++) {
             #pragma omp parallel for
             for (int j = 0; j < ctx_.num_bands(); j++) {
-                auto o = f(energy_fermi_ - this->get<T>(ik)->band_energy(j, ispn)) * ctx_.max_occupancy();
-                this->get<T>(ik)->band_occupancy(j, ispn, o);
+                auto o = f(energy_fermi_ - this->get<T>(it.i)->band_energy(j, ispn)) * ctx_.max_occupancy();
+                this->get<T>(it.i)->band_occupancy(j, ispn, o);
             }
         }
     }
@@ -421,28 +442,28 @@ void K_point_set::find_band_occupancies()
     }
 }
 
-template
-void K_point_set::find_band_occupancies<double>();
-#if defined(USE_FP32)
-template
-void K_point_set::find_band_occupancies<float>();
+template void
+K_point_set::find_band_occupancies<double>();
+#if defined(SIRIUS_USE_FP32)
+template void
+K_point_set::find_band_occupancies<float>();
 #endif
 
 template <typename T>
-double K_point_set::valence_eval_sum() const
+double
+K_point_set::valence_eval_sum() const
 {
     double eval_sum{0};
 
-    sddk::splindex<sddk::splindex_t::block> splb(ctx_.num_bands(), ctx_.comm_band().size(), ctx_.comm_band().rank());
+    splindex_block<> splb(ctx_.num_bands(), n_blocks(ctx_.comm_band().size()), block_id(ctx_.comm_band().rank()));
 
-    for (int ikloc = 0; ikloc < spl_num_kpoints_.local_size(); ikloc++) {
-        auto ik = spl_num_kpoints_[ikloc];
-        auto const& kp = this->get<T>(ik);
+    for (auto it : spl_num_kpoints_) {
+        auto const& kp = this->get<T>(it.i);
         double tmp{0};
         #pragma omp parallel for reduction(+:tmp)
         for (int j = 0; j < splb.local_size(); j++) {
             for (int ispn = 0; ispn < ctx_.num_spinors(); ispn++) {
-                tmp += kp->band_energy(splb[j], ispn) * kp->band_occupancy(splb[j], ispn);
+                tmp += kp->band_energy(splb.global_index(j), ispn) * kp->band_occupancy(splb.global_index(j), ispn);
             }
         }
         eval_sum += kp->weight() * tmp;
@@ -452,10 +473,11 @@ double K_point_set::valence_eval_sum() const
     return eval_sum;
 }
 
-double K_point_set::valence_eval_sum() const
+double
+K_point_set::valence_eval_sum() const
 {
     if (ctx_.cfg().parameters().precision_wf() == "fp32") {
-#if defined(USE_FP32)
+#if defined(SIRIUS_USE_FP32)
         return this->valence_eval_sum<float>();
 #else
         RTE_THROW("not compiled with FP32 support");
@@ -467,14 +489,14 @@ double K_point_set::valence_eval_sum() const
 }
 
 template <typename T>
-double K_point_set::entropy_sum() const
+double
+K_point_set::entropy_sum() const
 {
     double s_sum{0};
 
     double ne_target = ctx_.unit_cell().num_valence_electrons() - ctx_.cfg().parameters().extra_charge();
 
-    bool only_occ = (ctx_.num_mag_dims() != 1 &&
-                     std::abs(ctx_.num_bands() * ctx_.max_occupancy() - ne_target) < 1e-10);
+    bool only_occ = (ctx_.num_mag_dims() != 1 && std::abs(ctx_.num_bands() * ctx_.max_occupancy() - ne_target) < 1e-10);
 
     if (only_occ) {
         return 0;
@@ -482,16 +504,15 @@ double K_point_set::entropy_sum() const
 
     auto f = smearing::entropy(ctx_.smearing(), ctx_.smearing_width());
 
-    sddk::splindex<sddk::splindex_t::block> splb(ctx_.num_bands(), ctx_.comm_band().size(), ctx_.comm_band().rank());
+    splindex_block<> splb(ctx_.num_bands(), n_blocks(ctx_.comm_band().size()), block_id(ctx_.comm_band().rank()));
 
-    for (int ikloc = 0; ikloc < spl_num_kpoints_.local_size(); ikloc++) {
-        auto ik = spl_num_kpoints_[ikloc];
-        auto const& kp = this->get<T>(ik);
+    for (auto it : spl_num_kpoints_) {
+        auto const& kp = this->get<T>(it.i);
         double tmp{0};
         #pragma omp parallel for reduction(+:tmp)
         for (int j = 0; j < splb.local_size(); j++) {
             for (int ispn = 0; ispn < ctx_.num_spinors(); ispn++) {
-                tmp += ctx_.max_occupancy() * f(energy_fermi_ - kp->band_energy(splb[j], ispn));
+                tmp += ctx_.max_occupancy() * f(energy_fermi_ - kp->band_energy(splb.global_index(j), ispn));
             }
         }
         s_sum += kp->weight() * tmp;
@@ -501,10 +522,11 @@ double K_point_set::entropy_sum() const
     return s_sum;
 }
 
-double K_point_set::entropy_sum() const
+double
+K_point_set::entropy_sum() const
 {
     if (ctx_.cfg().parameters().precision_wf() == "fp32") {
-#if defined(USE_FP32)
+#if defined(SIRIUS_USE_FP32)
         return this->entropy_sum<float>();
 #else
         RTE_THROW("not compiled with FP32 support");
@@ -515,27 +537,28 @@ double K_point_set::entropy_sum() const
     }
 }
 
-void K_point_set::print_info()
+void
+K_point_set::print_info()
 {
     mpi::pstdout pout(this->comm());
 
     if (ctx_.comm().rank() == 0) {
         pout << std::endl;
         pout << "total number of k-points : " << num_kpoints() << std::endl;
-        pout << utils::hbar(80, '-') << std::endl;
+        pout << hbar(80, '-') << std::endl;
         pout << std::endl;
         pout << "  ik                vk                    weight  num_gkvec";
         if (ctx_.full_potential()) {
             pout << "   gklo_basis_size";
         }
-        pout << std::endl << utils::hbar(80, '-') << std::endl;
+        pout << std::endl << hbar(80, '-') << std::endl;
     }
 
-    for (int ikloc = 0; ikloc < spl_num_kpoints().local_size(); ikloc++) {
-        int ik = spl_num_kpoints(ikloc);
-        pout << std::setw(4) << ik << utils::ffmt(9, 4) << kpoints_[ik]->vk()[0] << utils::ffmt(9, 4)
-             << kpoints_[ik]->vk()[1] << utils::ffmt(9, 4) << kpoints_[ik]->vk()[2] << utils::ffmt(17, 6)
-             << kpoints_[ik]->weight() << std::setw(11) << kpoints_[ik]->num_gkvec();
+    for (auto it : spl_num_kpoints()) {
+        int ik = it.i;
+        pout << std::setw(4) << ik << ffmt(9, 4) << kpoints_[ik]->vk()[0] << ffmt(9, 4) << kpoints_[ik]->vk()[1]
+             << ffmt(9, 4) << kpoints_[ik]->vk()[2] << ffmt(17, 6) << kpoints_[ik]->weight() << std::setw(11)
+             << kpoints_[ik]->num_gkvec();
 
         if (ctx_.full_potential()) {
             pout << std::setw(18) << kpoints_[ik]->gklo_basis_size();
@@ -545,20 +568,21 @@ void K_point_set::print_info()
     RTE_OUT(ctx_.out()) << pout.flush(0);
 }
 
-void K_point_set::save(std::string const& name__) const
+void
+K_point_set::save(std::string const& name__) const
 {
     if (ctx_.comm().rank() == 0) {
-        if (!utils::file_exists(name__)) {
-            sddk::HDF5_tree(name__, sddk::hdf5_access_t::truncate);
+        if (!file_exists(name__)) {
+            HDF5_tree(name__, hdf5_access_t::truncate);
         }
-        sddk::HDF5_tree fout(name__, sddk::hdf5_access_t::read_write);
+        HDF5_tree fout(name__, hdf5_access_t::read_write);
         fout.create_node("K_point_set");
         fout["K_point_set"].write("num_kpoints", num_kpoints());
     }
     ctx_.comm().barrier();
     for (int ik = 0; ik < num_kpoints(); ik++) {
         /* check if this ranks stores the k-point */
-        if (ctx_.comm_k().rank() == spl_num_kpoints_.local_rank(ik)) {
+        if (ctx_.comm_k().rank() == spl_num_kpoints_.location(typename kp_index_t::global(ik)).ib) {
             this->get<double>(ik)->save(name__, ik);
         }
         /* wait for all */
@@ -567,9 +591,10 @@ void K_point_set::save(std::string const& name__) const
 }
 
 /// \todo check parameters of saved data in a separate function
-void K_point_set::load()
+void
+K_point_set::load()
 {
-    STOP();
+    RTE_THROW("not implemented");
 
     //== HDF5_tree fin(storage_file_name, false);
 
