@@ -46,13 +46,13 @@ class K_point_set
     /// List of k-points.
     std::vector<std::unique_ptr<K_point<double>>> kpoints_;
 
-#if defined(USE_FP32)
+#if defined(SIRIUS_USE_FP32)
     /// List of k-points in fp32 type, most calculation and assertion in this class only rely on fp64 type kpoints_
     std::vector<std::unique_ptr<K_point<float>>> kpoints_float_;
 #endif
 
     /// Split index of k-points.
-    sddk::splindex<sddk::splindex_t::chunk> spl_num_kpoints_;
+    splindex_chunk<kp_index_t> spl_num_kpoints_;
 
     /// Fermi energy which is searched in find_band_occupancies().
     double energy_fermi_{0};
@@ -64,17 +64,21 @@ class K_point_set
     K_point_set(K_point_set& src) = delete;
 
     /// Create regular grid of k-points.
-    void create_k_mesh(r3::vector<int> k_grid__, r3::vector<int> k_shift__, int use_symmetry__);
+    void
+    create_k_mesh(r3::vector<int> k_grid__, r3::vector<int> k_shift__, int use_symmetry__);
 
     bool initialized_{false};
 
     /// Return sum of valence eigen-values store in Kpoint<T>.
     template <typename T>
-    double valence_eval_sum() const;
+    double
+    valence_eval_sum() const;
 
     /// Return entropy contribution from smearing store in Kpoint<T>.
     template <typename T>
-    double entropy_sum() const;
+    double
+    entropy_sum() const;
+
   public:
     /// Create empty k-point set.
     K_point_set(Simulation_context& ctx__)
@@ -106,80 +110,100 @@ class K_point_set
     }
 
     /// Initialize the k-point set
-    void initialize(std::vector<int> const& counts = {});
+    void
+    initialize(std::vector<int> const& counts = {});
 
     /// Sync band energies or occupancies between all MPI ranks.
     template <typename T, sync_band_t what>
-    void sync_band();
+    void
+    sync_band();
 
     /// Find Fermi energy and band occupation numbers.
     template <typename T>
-    void find_band_occupancies();
+    void
+    find_band_occupancies();
 
     /// Print basic info to the standard output.
-    void print_info();
+    void
+    print_info();
 
     /// Save k-point set to HDF5 file.
-    void save(std::string const& name__) const;
+    void
+    save(std::string const& name__) const;
 
-    void load();
+    void
+    load();
 
     /// Return sum of valence eigen-values.
-    double valence_eval_sum() const;
+    double
+    valence_eval_sum() const;
 
     /// Return entropy contribution from smearing.
-    double entropy_sum() const;
+    double
+    entropy_sum() const;
 
-    inline auto const& spl_num_kpoints() const
+    inline auto const&
+    spl_num_kpoints() const
     {
         return spl_num_kpoints_;
     }
 
-    inline auto const& comm() const
+    inline auto const&
+    comm() const
     {
         return ctx_.comm_k();
     }
 
     /// Update k-points after moving atoms or changing the lattice vectors.
-    void update()
+    void
+    update()
     {
         /* update k-points */
-        for (int ikloc = 0; ikloc < spl_num_kpoints().local_size(); ikloc++) {
-            int ik = spl_num_kpoints(ikloc);
-            kpoints_[ik]->update();
-#if defined(USE_FP32)
-            kpoints_float_[ik]->update();
+        for (auto it : spl_num_kpoints_) {
+            kpoints_[it.i]->update();
+#if defined(SIRIUS_USE_FP32)
+            kpoints_float_[it.i]->update();
 #endif
         }
     }
 
     /// Get a list of band energies for a given k-point index.
     template <typename T>
-    std::vector<double> get_band_energies(int ik__, int ispn__) const;
+    auto
+    get_band_energies(int ik__, int ispn__) const
+    {
+        std::vector<double> bnd_e(ctx_.num_bands());
+        for (int j = 0; j < ctx_.num_bands(); j++) {
+            bnd_e[j] = (*this).get<T>(ik__)->band_energy(j, ispn__);
+        }
+        return bnd_e;
+    }
 
     /// Return maximum number of G+k vectors among all k-points.
-    int max_num_gkvec() const
+    int
+    max_num_gkvec() const
     {
         int max_num_gkvec{0};
-        for (int ikloc = 0; ikloc < spl_num_kpoints_.local_size(); ikloc++) {
-            auto ik       = spl_num_kpoints_[ikloc];
-            max_num_gkvec = std::max(max_num_gkvec, kpoints_[ik]->num_gkvec());
+        for (auto it : spl_num_kpoints_) {
+            max_num_gkvec = std::max(max_num_gkvec, kpoints_[it.i]->num_gkvec());
         }
         comm().allreduce<int, mpi::op_t::max>(&max_num_gkvec, 1);
         return max_num_gkvec;
     }
 
     /// Add k-point to the set.
-    void add_kpoint(r3::vector<double> vk__, double weight__)
+    void
+    add_kpoint(r3::vector<double> vk__, double weight__)
     {
         kpoints_.push_back(std::unique_ptr<K_point<double>>(new K_point<double>(ctx_, vk__, weight__)));
-#ifdef USE_FP32
+#ifdef SIRIUS_USE_FP32
         kpoints_float_.push_back(std::unique_ptr<K_point<float>>(new K_point<float>(ctx_, vk__, weight__)));
 #endif
     }
 
     /// Add multiple k-points to the set.
-    void add_kpoints(sddk::mdarray<double, 2> const& kpoints__, double const* weights__)
+    void
+    add_kpoints(mdarray<double, 2> const& kpoints__, double const* weights__)
     {
         for (int ik = 0; ik < (int)kpoints__.size(1); ik++) {
             add_kpoint(&kpoints__(0, ik), weights__[ik]);
@@ -187,37 +211,50 @@ class K_point_set
     }
 
     template <typename T>
-    inline K_point<T>* get(int ik__) const;
+    inline K_point<T>*
+    get(int ik__) const;
 
     template <typename T>
-    inline K_point<T>* get(int ik__)
+    inline K_point<T>*
+    get(int ik__)
     {
         return const_cast<K_point<T>*>(static_cast<K_point_set const&>(*this).get<T>(ik__));
     }
 
     /// Return total number of k-points.
-    inline int num_kpoints() const
+    inline int
+    num_kpoints() const
     {
         return static_cast<int>(kpoints_.size());
     }
 
-    inline int spl_num_kpoints(int ikloc) const
+    inline auto
+    spl_num_kpoints(kp_index_t::local ikloc__) const
     {
-        return spl_num_kpoints_[ikloc];
+        return spl_num_kpoints_.global_index(ikloc__);
     }
 
-    inline double energy_fermi() const
+    inline double
+    energy_fermi() const
     {
         return energy_fermi_;
     }
 
-    inline double band_gap() const
+    inline void
+    set_energy_fermi(double energy_fermi__)
+    {
+        this->energy_fermi_ = energy_fermi__;
+    }
+
+    inline double
+    band_gap() const
     {
         return band_gap_;
     }
 
     /// Find index of k-point.
-    inline int find_kpoint(r3::vector<double> vk__)
+    inline int
+    find_kpoint(r3::vector<double> vk__)
     {
         for (int ik = 0; ik < num_kpoints(); ik++) {
             if ((kpoints_[ik]->vk() - vk__).length() < 1e-12) {
@@ -227,25 +264,28 @@ class K_point_set
         return -1;
     }
 
-    inline auto& ctx()
+    inline auto&
+    ctx()
     {
         return ctx_;
     }
 
-    const auto& unit_cell()
+    const auto&
+    unit_cell()
     {
         return ctx_.unit_cell();
     }
 
     /// Send G+k vectors of k-point jk to a given rank.
     /** Other ranks receive an empty Gvec placeholder */
-    inline fft::Gvec get_gkvec(int jk__, int rank__)
+    inline fft::Gvec
+    get_gkvec(kp_index_t::global jk__, int rank__)
     {
         /* rank in the k-point communicator */
         int my_rank = comm().rank();
 
         /* rank that stores jk */
-        int jrank = spl_num_kpoints().local_rank(jk__);
+        int jrank = spl_num_kpoints().location(jk__).ib;
 
         /* need this to pass communicator */
         fft::Gvec gkvec(ctx_.comm_band());
@@ -261,18 +301,20 @@ class K_point_set
     }
 };
 
-template<>
-inline K_point<double>* K_point_set::get<double>(int ik__) const
+template <>
+inline K_point<double>*
+K_point_set::get<double>(int ik__) const
 {
-    assert(ik__ >= 0 && ik__ < (int)kpoints_.size());
+    RTE_ASSERT(ik__ >= 0 && ik__ < (int)kpoints_.size());
     return kpoints_[ik__].get();
 }
 
-template<>
-inline K_point<float>* K_point_set::get<float>(int ik__) const
+template <>
+inline K_point<float>*
+K_point_set::get<float>(int ik__) const
 {
-#if defined(USE_FP32)
-    assert(ik__ >= 0 && ik__ < (int)kpoints_float_.size());
+#if defined(SIRIUS_USE_FP32)
+    RTE_ASSERT(ik__ >= 0 && ik__ < (int)kpoints_float_.size());
     return kpoints_float_[ik__].get();
 #else
     RTE_THROW("not compiled with FP32 support");
@@ -280,16 +322,6 @@ inline K_point<float>* K_point_set::get<float>(int ik__) const
 #endif
 }
 
-template <typename T>
-std::vector<double> K_point_set::get_band_energies(int ik__, int ispn__) const
-{
-    std::vector<double> bnd_e(ctx_.num_bands());
-    for (int j = 0; j < ctx_.num_bands(); j++) {
-        bnd_e[j] = (*this).get<T>(ik__)->band_energy(j, ispn__);
-    }
-    return bnd_e;
-}
-
 }; // namespace sirius
 
 #endif // __K_POINT_SET_H__
