@@ -27,7 +27,7 @@
 
 #include <gsl/gsl_sf_bessel.h>
 #include "unit_cell/unit_cell.hpp"
-#include "fft/gvec.hpp"
+#include "core/fft/gvec.hpp"
 
 namespace sirius {
 
@@ -62,10 +62,10 @@ class Matching_coefficients // TODO: compute on GPU
     std::vector<double> gkvec_len_;
 
     /// Spherical harmonics Ylm(theta, phi) of the G+k vectors.
-    sddk::mdarray<std::complex<double>, 2> gkvec_ylm_;
+    mdarray<std::complex<double>, 2> gkvec_ylm_;
 
     /// Precomputed values for the linear equations for matching coefficients.
-    sddk::mdarray<std::complex<double>, 4> alm_b_;
+    mdarray<std::complex<double>, 4> alm_b_;
 
     /// Generate matching coefficients for a specific \f$ \ell \f$ and order.
     /** \param [in] ngk           Number of G+k vectors.
@@ -78,8 +78,9 @@ class Matching_coefficients // TODO: compute on GPU
      *  \param [out] alm          Pointer to alm coefficients.
      */
     template <int N, bool conjugate, typename T, typename = std::enable_if_t<!std::is_scalar<T>::value>>
-    inline void generate(int ngk, std::vector<std::complex<double>> const& phase_factors__, int iat, int l, int lm,
-            int nu, r3::matrix<double> const& A, T* alm) const
+    inline void
+    generate(int ngk, std::vector<std::complex<double>> const& phase_factors__, int iat, int l, int lm, int nu,
+             r3::matrix<double> const& A, T* alm) const
     {
         std::complex<double> zt;
 
@@ -114,9 +115,9 @@ class Matching_coefficients // TODO: compute on GPU
         , gkvec_(gkvec__)
     {
         int lmax_apw  = unit_cell__.lmax_apw();
-        int lmmax_apw = utils::lmmax(lmax_apw);
+        int lmmax_apw = sf::lmmax(lmax_apw);
 
-        gkvec_ylm_ = sddk::mdarray<std::complex<double>, 2>(gkvec_.count(), lmmax_apw);
+        gkvec_ylm_ = mdarray<std::complex<double>, 2>({gkvec_.count(), lmmax_apw});
         gkvec_len_.resize(gkvec_.count());
 
         /* get length and Ylm harmonics of G+k vectors */
@@ -126,9 +127,9 @@ class Matching_coefficients // TODO: compute on GPU
 
             #pragma omp for
             for (int i = 0; i < gkvec_.count(); i++) {
-                auto gkvec_cart = gkvec_.gkvec_cart<sddk::index_domain_t::local>(i);
+                auto gkvec_cart = gkvec_.gkvec_cart<index_domain_t::local>(i);
                 /* get r, theta, phi */
-                auto vs = r3::spherical_coordinates(gkvec_cart);
+                auto vs       = r3::spherical_coordinates(gkvec_cart);
                 gkvec_len_[i] = vs[0];
 
                 /* get spherical harmonics */
@@ -140,13 +141,13 @@ class Matching_coefficients // TODO: compute on GPU
             }
         }
 
-        alm_b_ = sddk::mdarray<std::complex<double>, 4>(3, gkvec_.count(), lmax_apw + 1, unit_cell_.num_atom_types());
+        alm_b_ = mdarray<std::complex<double>, 4>({3, gkvec_.count(), lmax_apw + 1, unit_cell_.num_atom_types()});
         alm_b_.zero();
 
         #pragma omp parallel
         {
             /* value and first two derivatives of spherical Bessel functions */
-            sddk::mdarray<double, 2> sbessel_mt(lmax_apw + 2, 3);
+            mdarray<double, 2> sbessel_mt({lmax_apw + 2, 3});
 
             #pragma omp for
             for (int igk = 0; igk < gkvec_.count(); igk++) {
@@ -174,7 +175,7 @@ class Matching_coefficients // TODO: compute on GPU
                     }
 
                     for (int l = 0; l <= lmax_apw; l++) {
-                        std::complex<double> z       = std::pow(std::complex<double>(0, 1), l);
+                        std::complex<double> z = std::pow(std::complex<double>(0, 1), l);
                         double f               = fourpi / std::sqrt(unit_cell_.omega());
                         alm_b_(0, igk, l, iat) = z * f * sbessel_mt(l, 0);
                         alm_b_(1, igk, l, iat) = z * f * sbessel_mt(l, 1);
@@ -190,17 +191,18 @@ class Matching_coefficients // TODO: compute on GPU
         \param [out] alm       Array of matching coefficients with dimension indices \f$ ({\bf G+k}, \xi) \f$.
      */
     template <bool conjugate, typename T, typename = std::enable_if_t<!std::is_scalar<T>::value>>
-    void generate(Atom const& atom__, sddk::mdarray<T, 2>& alm__) const
+    void
+    generate(Atom const& atom__, mdarray<T, 2>& alm__) const
     {
         auto& type = atom__.type();
 
-        assert(type.max_aw_order() <= 3);
+        RTE_ASSERT(type.max_aw_order() <= 3);
 
         int iat = type.id();
 
         std::vector<std::complex<double>> phase_factors(gkvec_.count());
         for (int i = 0; i < gkvec_.count(); i++) {
-            double phase     = twopi * dot(gkvec_.template gkvec<sddk::index_domain_t::local>(i), atom__.position());
+            double phase     = twopi * dot(gkvec_.template gkvec<index_domain_t::local>(i), atom__.position());
             phase_factors[i] = std::exp(std::complex<double>(0, phase));
         }
 
@@ -227,7 +229,7 @@ class Matching_coefficients // TODO: compute on GPU
                             s << "Ill defined plane wave matching problem for atom type " << iat << ", l = " << l
                               << std::endl
                               << "  radial function value at the MT boundary : " << A(0, 0);
-                            WARNING(s.str());
+                            RTE_WARNING(s.str());
                         }
                     }
 
@@ -243,7 +245,7 @@ class Matching_coefficients // TODO: compute on GPU
                             s << "Ill defined plane wave matching problem for atom type " << iat << ", l = " << l
                               << std::endl
                               << "  radial function value at the MT boundary : " << A(0, 0);
-                            WARNING(s.str());
+                            RTE_WARNING(s.str());
                         }
                     }
 
@@ -263,7 +265,7 @@ class Matching_coefficients // TODO: compute on GPU
         }
 
         for (int xi = 0; xi < type.mt_aw_basis_size(); xi++) {
-            int l  = type.indexb(xi).l;
+            int l  = type.indexb(xi).am.l();
             int lm = type.indexb(xi).lm;
             int nu = type.indexb(xi).order;
 
@@ -287,13 +289,14 @@ class Matching_coefficients // TODO: compute on GPU
                     break;
                 }
                 default: {
-                    TERMINATE("wrong order of augmented wave");
+                    RTE_THROW("wrong order of augmented wave");
                 }
             }
         }
     }
 
-    auto const& gkvec() const
+    auto const&
+    gkvec() const
     {
         return gkvec_;
     }
