@@ -26,24 +26,30 @@
 #define __MIXER_FUNCTIONS_HPP__
 
 #include "function3d/periodic_function.hpp"
-#include "SDDK/memory.hpp"
+#include "core/memory.hpp"
 #include "mixer/mixer.hpp"
-#include "density/paw_density.hpp"
 #include "hubbard/hubbard_matrix.hpp"
+#include "density/density_matrix.hpp"
+#include "density/density.hpp"
 
 namespace sirius {
 
 namespace mixer {
 
-FunctionProperties<Periodic_function<double>> periodic_function_property();
+FunctionProperties<Periodic_function<double>>
+periodic_function_property();
 
-FunctionProperties<Periodic_function<double>> periodic_function_property_modified(bool use_coarse_gvec__);
+FunctionProperties<Periodic_function<double>>
+periodic_function_property_modified(bool use_coarse_gvec__);
 
-FunctionProperties<sddk::mdarray<std::complex<double>, 4>> density_function_property();
+FunctionProperties<density_matrix_t>
+density_function_property();
 
-FunctionProperties<paw_density> paw_density_function_property();
+FunctionProperties<PAW_density<double>>
+paw_density_function_property();
 
-FunctionProperties<Hubbard_matrix> hubbard_matrix_function_property();
+FunctionProperties<Hubbard_matrix>
+hubbard_matrix_function_property();
 
 } // namespace mixer
 
