@@ -24,11 +24,16 @@
 
 #include "potential.hpp"
 #include "xc_functional.hpp"
+#include "lapw/generate_gvec_ylm.hpp"
+#include "lapw/generate_sbessel_mt.hpp"
+#include "symmetry/symmetrize_field4d.hpp"
 
 namespace sirius {
 
 Potential::Potential(Simulation_context& ctx__)
-    : Field4D(ctx__, ctx__.lmmax_pot())
+    : Field4D(ctx__, lmax_t(ctx__.lmax_pot()),
+              {ctx__.periodic_function_ptr("veff"), ctx__.periodic_function_ptr("bz"),
+               ctx__.periodic_function_ptr("bx"), ctx__.periodic_function_ptr("by")})
     , unit_cell_(ctx__.unit_cell())
     , comm_(ctx__.comm())
     , hubbard_potential_(ctx__)
@@ -36,26 +41,32 @@ Potential::Potential(Simulation_context& ctx__)
     PROFILE("sirius::Potential");
 
     if (!ctx_.initialized()) {
-        TERMINATE("Simulation_context is not initialized");
+        RTE_THROW("Simulation_context is not initialized");
     }
 
-    lmax_ = std::max(ctx_.lmax_rho(), ctx_.lmax_pot());
+    int lmax{-1};
 
-    if (lmax_ >= 0) {
-        sht_  = std::unique_ptr<SHT>(new SHT(ctx_.processing_unit(), lmax_, ctx_.cfg().settings().sht_coverage()));
-        if (ctx_.cfg().control().verification() >= 1)  {
+    if (ctx_.full_potential()) {
+        lmax = std::max(ctx_.lmax_rho(), ctx_.lmax_pot());
+    } else {
+        lmax = 2 * ctx_.unit_cell().lmax();
+    }
+
+    if (lmax >= 0) {
+        sht_ = std::make_unique<SHT>(ctx_.processing_unit(), lmax, ctx_.cfg().settings().sht_coverage());
+        if (ctx_.cfg().control().verification() >= 1) {
             sht_->check();
         }
-        l_by_lm_ = utils::l_by_lm(lmax_);
+        l_by_lm_ = sf::l_by_lm(lmax);
 
         /* precompute i^l */
-        zil_.resize(lmax_ + 1);
-        for (int l = 0; l <= lmax_; l++) {
+        zil_.resize(lmax + 1);
+        for (int l = 0; l <= lmax; l++) {
             zil_[l] = std::pow(std::complex<double>(0, 1), l);
         }
 
-        zilm_.resize(utils::lmmax(lmax_));
-        for (int l = 0, lm = 0; l <= lmax_; l++) {
+        zilm_.resize(sf::lmmax(lmax));
+        for (int l = 0, lm = 0; l <= lmax; l++) {
             for (int m = -l; m <= l; m++, lm++) {
                 zilm_[lm] = zil_[l];
             }
@@ -64,56 +75,62 @@ Potential::Potential(Simulation_context& ctx__)
 
     /* create list of XC functionals */
     for (auto& xc_label : ctx_.xc_functionals()) {
-        xc_func_.emplace_back(XC_functional(ctx_.spfft<double>(), ctx_.unit_cell().lattice_vectors(), xc_label, ctx_.num_spins()));
+        xc_func_.emplace_back(
+                XC_functional(ctx_.spfft<double>(), ctx_.unit_cell().lattice_vectors(), xc_label, ctx_.num_spins()));
         if (ctx_.cfg().parameters().xc_dens_tre() > 0) {
             xc_func_.back().set_dens_threshold(ctx_.cfg().parameters().xc_dens_tre());
         }
     }
 
-    using pf = Periodic_function<double>;
+    using pf  = Periodic_function<double>;
     using spf = Smooth_periodic_function<double>;
 
-    hartree_potential_ = std::unique_ptr<pf>(new pf(ctx_, ctx_.lmmax_pot()));
-    hartree_potential_->allocate_mt(false);
-
-    xc_potential_ = std::unique_ptr<pf>(new pf(ctx_, ctx_.lmmax_pot()));
-    xc_potential_->allocate_mt(false);
-
-    xc_energy_density_ = std::unique_ptr<pf>(new pf(ctx_, ctx_.lmmax_pot()));
-    xc_energy_density_->allocate_mt(false);
+    if (ctx_.full_potential()) {
+        hartree_potential_ = std::make_unique<pf>(
+                ctx_, [&](int ia) { return lmax_t(ctx_.lmax_pot()); }, &ctx_.unit_cell().spl_num_atoms());
+        xc_potential_ = std::make_unique<pf>(
+                ctx_, [&](int ia) { return lmax_t(ctx_.lmax_pot()); }, &ctx_.unit_cell().spl_num_atoms());
+        xc_energy_density_ = std::make_unique<pf>(
+                ctx_, [&](int ia) { return lmax_t(ctx_.lmax_pot()); }, &ctx_.unit_cell().spl_num_atoms());
+    } else {
+        hartree_potential_ = std::make_unique<pf>(ctx_);
+        xc_potential_      = std::make_unique<pf>(ctx_);
+        xc_energy_density_ = std::make_unique<pf>(ctx_);
+    }
 
     if (this->is_gradient_correction()) {
         int nsigma = (ctx_.num_spins() == 1) ? 1 : 3;
-        for (int i = 0; i < nsigma ; i++) {
-            vsigma_[i] = std::unique_ptr<spf>(new spf(ctx_.spfft<double>(), ctx_.gvec_fft_sptr()));
+        for (int i = 0; i < nsigma; i++) {
+            vsigma_[i] = std::make_unique<spf>(ctx_.spfft<double>(), ctx_.gvec_fft_sptr());
         }
     }
 
     if (!ctx_.full_potential()) {
-        local_potential_ = std::unique_ptr<spf>(new spf(ctx_.spfft<double>(), ctx_.gvec_fft_sptr()));
-        dveff_ = std::unique_ptr<spf>(new spf(ctx_.spfft<double>(), ctx_.gvec_fft_sptr()));
+        local_potential_ = std::make_unique<spf>(ctx_.spfft<double>(), ctx_.gvec_fft_sptr());
+        dveff_           = std::make_unique<spf>(ctx_.spfft<double>(), ctx_.gvec_fft_sptr());
         dveff_->zero();
     }
 
-    vh_el_ = sddk::mdarray<double, 1>(unit_cell_.num_atoms());
+    vh_el_ = mdarray<double, 1>({unit_cell_.num_atoms()});
 
     if (ctx_.full_potential()) {
-        gvec_ylm_ = sddk::mdarray<std::complex<double>, 2>(ctx_.lmmax_pot(), ctx_.gvec().count(), sddk::memory_t::host, "gvec_ylm_");
+        gvec_ylm_ =
+                mdarray<std::complex<double>, 2>({ctx_.lmmax_pot(), ctx_.gvec().count()}, mdarray_label("gvec_ylm_"));
 
         switch (ctx_.valence_relativity()) {
             case relativity_t::iora: {
-                rm2_inv_pw_ = sddk::mdarray<std::complex<double>, 1>(ctx_.gvec().num_gvec());
+                rm2_inv_pw_ = mdarray<std::complex<double>, 1>({ctx_.gvec().num_gvec()});
             }
             case relativity_t::zora: {
-                rm_inv_pw_ = sddk::mdarray<std::complex<double>, 1>(ctx_.gvec().num_gvec());
+                rm_inv_pw_ = mdarray<std::complex<double>, 1>({ctx_.gvec().num_gvec()});
             }
             default: {
-                veff_pw_ = sddk::mdarray<std::complex<double>, 1>(ctx_.gvec().num_gvec());
+                veff_pw_ = mdarray<std::complex<double>, 1>({ctx_.gvec().num_gvec()});
             }
         }
     }
 
-    aux_bf_ = sddk::mdarray<double, 2>(3, ctx_.unit_cell().num_atoms());
+    aux_bf_ = mdarray<double, 2>({3, ctx_.unit_cell().num_atoms()});
     aux_bf_.zero();
 
     if (ctx_.cfg().parameters().reduce_aux_bf() > 0 && ctx_.cfg().parameters().reduce_aux_bf() < 1) {
@@ -134,7 +151,8 @@ Potential::Potential(Simulation_context& ctx__)
     update();
 }
 
-void Potential::update()
+void
+Potential::update()
 {
     PROFILE("sirius::Potential::update");
 
@@ -142,8 +160,10 @@ void Potential::update()
         local_potential_->zero();
         generate_local_potential();
     } else {
-        gvec_ylm_ = ctx_.generate_gvec_ylm(ctx_.lmax_pot());
-        sbessel_mt_ = ctx_.generate_sbessel_mt(lmax_ + pseudo_density_order_ + 1);
+        gvec_ylm_ = generate_gvec_ylm(ctx_, ctx_.lmax_pot());
+
+        auto lmax   = std::max(ctx_.lmax_rho(), ctx_.lmax_pot());
+        sbessel_mt_ = generate_sbessel_mt(ctx_, lmax + pseudo_density_order_ + 1);
 
         /* compute moments of spherical Bessel functions
          *
@@ -152,10 +172,8 @@ void Potential::update()
          *
          * and use relation between Bessel and spherical Bessel functions:
          * Subscript[j, n](z)=Sqrt[\[Pi]/2]/Sqrt[z]Subscript[J, n+1/2](z) */
-        sbessel_mom_ = sddk::mdarray<double, 3>(ctx_.lmax_rho() + 1,
-                                          ctx_.gvec().count(),
-                                          unit_cell_.num_atom_types(),
-                                          sddk::memory_t::host, "sbessel_mom_");
+        sbessel_mom_ = mdarray<double, 3>({ctx_.lmax_rho() + 1, ctx_.gvec().count(), unit_cell_.num_atom_types()},
+                                          mdarray_label("sbessel_mom_"));
         sbessel_mom_.zero();
         int ig0{0};
         if (ctx_.comm().rank() == 0) {
@@ -168,7 +186,7 @@ void Potential::update()
         for (int iat = 0; iat < unit_cell_.num_atom_types(); iat++) {
             #pragma omp parallel for schedule(static)
             for (int igloc = ig0; igloc < ctx_.gvec().count(); igloc++) {
-                auto len = ctx_.gvec().gvec_cart<sddk::index_domain_t::local>(igloc).length();
+                auto len = ctx_.gvec().gvec_cart<index_domain_t::local>(igloc).length();
                 for (int l = 0; l <= ctx_.lmax_rho(); l++) {
                     sbessel_mom_(l, igloc, iat) = std::pow(unit_cell_.atom_type(iat).mt_radius(), l + 2) *
                                                   sbessel_mt_(l + 1, igloc, iat) / len;
@@ -179,8 +197,8 @@ void Potential::update()
         /* compute Gamma[5/2 + n + l] / Gamma[3/2 + l] / R^l
          *
          * use Gamma[1/2 + p] = (2p - 1)!!/2^p Sqrt[Pi] */
-        gamma_factors_R_ = sddk::mdarray<double, 2>(ctx_.lmax_rho() + 1, unit_cell_.num_atom_types(),
-                sddk::memory_t::host, "gamma_factors_R_");
+        gamma_factors_R_ = mdarray<double, 2>({ctx_.lmax_rho() + 1, unit_cell_.num_atom_types()},
+                                              mdarray_label("gamma_factors_R_"));
         for (int iat = 0; iat < unit_cell_.num_atom_types(); iat++) {
             for (int l = 0; l <= ctx_.lmax_rho(); l++) {
                 long double Rl = std::pow(unit_cell_.atom_type(iat).mt_radius(), l);
@@ -210,7 +228,8 @@ void Potential::update()
     }
 }
 
-bool Potential::is_gradient_correction() const
+bool
+Potential::is_gradient_correction() const
 {
     bool is_gga{false};
     for (auto& ixc : xc_func_) {
@@ -221,14 +240,15 @@ bool Potential::is_gradient_correction() const
     return is_gga;
 }
 
-void Potential::generate(Density const& density__, bool use_symmetry__, bool transform_to_rg__)
+void
+Potential::generate(Density const& density__, bool use_symmetry__, bool transform_to_rg__)
 {
     PROFILE("sirius::Potential::generate");
 
     if (!ctx_.full_potential()) {
         /* save current effective potential */
-        for (size_t ig = 0; ig < effective_potential().f_pw_local().size(); ig++) {
-            dveff_->f_pw_local(ig) = effective_potential().f_pw_local(ig);
+        for (size_t ig = 0; ig < effective_potential().rg().f_pw_local().size(); ig++) {
+            dveff_->f_pw_local(ig) = effective_potential().rg().f_pw_local(ig);
         }
     }
 
@@ -238,48 +258,44 @@ void Potential::generate(Density const& density__, bool use_symmetry__, bool tra
     auto veff_callback = ctx_.veff_callback();
     if (veff_callback) {
         veff_callback();
-        //if (!ctx_.full_potential()) {
-        //    /* add local ionic potential to the effective potential */
-        //    effective_potential().add(local_potential());
-        //}
+        // if (!ctx_.full_potential()) {
+        //     /* add local ionic potential to the effective potential */
+        //     effective_potential().add(local_potential());
+        // }
         /* transform to real space */
-        //fft_transform(1);
+        // fft_transform(1);
     } else {
         /* solve Poisson equation */
         poisson(density__.rho());
 
         /* add Hartree potential to the total potential */
-        effective_potential().add(hartree_potential());
+        effective_potential() += hartree_potential();
 
-        if (ctx_.cfg().control().print_hash()) {
-            auto h = effective_potential().hash_f_rg();
-            if (ctx_.comm().rank() == 0) {
-                utils::print_hash("Vha", h);
-            }
+        if (env::print_hash()) {
+            auto h = effective_potential().rg().hash_f_rg();
+            print_hash("Vha", h, ctx_.out());
         }
 
         if (ctx_.full_potential()) {
             xc(density__);
         } else {
             /* add local ionic potential to the effective potential */
-            effective_potential().add(local_potential());
+            effective_potential().rg() += local_potential();
             /* construct XC potentials from rho + rho_core */
             xc<true>(density__);
         }
         /* add XC potential to the effective potential */
-        effective_potential().add(xc_potential());
+        effective_potential() += xc_potential();
 
-        if (ctx_.cfg().control().print_hash()) {
-            auto h = effective_potential().hash_f_rg();
-            if (ctx_.comm().rank() == 0) {
-                utils::print_hash("Vha+Vxc", h);
-            }
+        if (env::print_hash()) {
+            auto h = effective_potential().rg().hash_f_rg();
+            print_hash("Vha+Vxc", h, ctx_.out());
         }
 
         if (ctx_.full_potential()) {
-            effective_potential().sync_mt();
+            effective_potential().mt().sync(ctx_.unit_cell().spl_num_atoms());
             for (int j = 0; j < ctx_.num_mag_dims(); j++) {
-                effective_magnetic_field(j).sync_mt();
+                effective_magnetic_field(j).mt().sync(ctx_.unit_cell().spl_num_atoms());
             }
         }
 
@@ -293,7 +309,7 @@ void Potential::generate(Density const& density__, bool use_symmetry__, bool tra
 
     if (use_symmetry__) {
         /* symmetrize potential and effective magnetic field */
-        this->symmetrize();
+        symmetrize_field4d(*this);
         if (transform_to_rg__) {
             /* transform potential to real space after symmetrization */
             this->fft_transform(1);
@@ -302,21 +318,53 @@ void Potential::generate(Density const& density__, bool use_symmetry__, bool tra
 
     if (!ctx_.full_potential()) {
         /* this is needed later to compute scf correction to forces */
-        for (size_t ig = 0; ig < effective_potential().f_pw_local().size(); ig++) {
-            dveff_->f_pw_local(ig) = effective_potential().f_pw_local(ig) - dveff_->f_pw_local(ig);
+        for (size_t ig = 0; ig < effective_potential().rg().f_pw_local().size(); ig++) {
+            dveff_->f_pw_local(ig) = effective_potential().rg().f_pw_local(ig) - dveff_->f_pw_local(ig);
         }
     }
 
-    if (ctx_.cfg().control().print_hash()) {
-        auto h = effective_potential().hash_f_pw();
-        if (ctx_.comm().rank() == 0) {
-            utils::print_hash("V(G)", h);
-        }
+    if (env::print_hash()) {
+        auto h = effective_potential().rg().hash_f_pw();
+        print_hash("V(G)", h, ctx_.out());
     }
 
     if (!ctx_.full_potential()) {
         generate_D_operator_matrix();
         generate_PAW_effective_potential(density__);
+        if (ctx_.verbosity() >= 3) {
+            rte::ostream out(ctx_.out(), "potential");
+            out << "density matrix" << std::endl;
+            for (int ia = 0; ia < ctx_.unit_cell().num_atoms(); ia++) {
+                auto& atom = ctx_.unit_cell().atom(ia);
+                out << "atom : " << ia << std::endl;
+                for (int imagn = 0; imagn < ctx_.num_mag_comp(); imagn++) {
+                    out << "  imagn : " << imagn << std::endl;
+                    for (int ib2 = 0; ib2 < atom.mt_basis_size(); ib2++) {
+                        out << "    ";
+                        for (int ib1 = 0; ib1 < atom.mt_basis_size(); ib1++) {
+                            out << ffmt(8, 3) << density__.density_matrix(ia)(ib1, ib2, imagn);
+                        }
+                        out << std::endl;
+                    }
+                }
+            }
+
+            out << "D operator matrix" << std::endl;
+            for (int ia = 0; ia < ctx_.unit_cell().num_atoms(); ia++) {
+                auto& atom = ctx_.unit_cell().atom(ia);
+                out << "atom : " << ia << std::endl;
+                for (int imagn = 0; imagn < ctx_.num_mag_dims() + 1; imagn++) {
+                    out << "  imagn : " << imagn << std::endl;
+                    for (int ib2 = 0; ib2 < atom.mt_basis_size(); ib2++) {
+                        out << "    ";
+                        for (int ib1 = 0; ib1 < atom.mt_basis_size(); ib1++) {
+                            out << ffmt(8, 3) << atom.d_mtrx(ib1, ib2, imagn);
+                        }
+                        out << std::endl;
+                    }
+                }
+            }
+        }
     }
 
     if (ctx_.hubbard_correction()) {
@@ -332,16 +380,15 @@ void Potential::generate(Density const& density__, bool use_symmetry__, bool tra
     }
 }
 
-void Potential::save()
+void
+Potential::save(std::string name__)
 {
-    effective_potential().hdf5_write(storage_file_name, "effective_potential");
+    effective_potential().hdf5_write(name__, "effective_potential");
     for (int j = 0; j < ctx_.num_mag_dims(); j++) {
-        std::stringstream s;
-        s << "effective_magnetic_field/" << j;
-        effective_magnetic_field(j).hdf5_write(storage_file_name, s.str());
+        effective_magnetic_field(j).hdf5_write(name__, "effective_magnetic_field/" + std::to_string(j));
     }
     if (ctx_.comm().rank() == 0 && !ctx_.full_potential()) {
-        sddk::HDF5_tree fout(storage_file_name, sddk::hdf5_access_t::read_write);
+        HDF5_tree fout(name__, hdf5_access_t::read_write);
         for (int j = 0; j < ctx_.unit_cell().num_atoms(); j++) {
             if (ctx_.unit_cell().atom(j).mt_basis_size() != 0) {
                 fout["unit_cell"]["atoms"][j].write("D_operator", ctx_.unit_cell().atom(j).d_mtrx());
@@ -351,22 +398,23 @@ void Potential::save()
     comm_.barrier();
 }
 
-void Potential::load()
+void
+Potential::load(std::string name__)
 {
-    sddk::HDF5_tree fin(storage_file_name, sddk::hdf5_access_t::read_only);
+    HDF5_tree fin(name__, hdf5_access_t::read_only);
 
     int ngv;
     fin.read("/parameters/num_gvec", &ngv, 1);
     if (ngv != ctx_.gvec().num_gvec()) {
-        TERMINATE("wrong number of G-vectors");
+        RTE_THROW("wrong number of G-vectors");
     }
-    sddk::mdarray<int, 2> gv(3, ngv);
+    mdarray<int, 2> gv({3, ngv});
     fin.read("/parameters/gvec", gv);
 
-    effective_potential().hdf5_read(fin["effective_potential"], gv);
+    effective_potential().hdf5_read(name__, "effective_potential", gv);
 
     for (int j = 0; j < ctx_.num_mag_dims(); j++) {
-        effective_magnetic_field(j).hdf5_read(fin["effective_magnetic_field"][j], gv);
+        effective_magnetic_field(j).hdf5_read(name__, "effective_magnetic_field/" + std::to_string(j), gv);
     }
 
     if (ctx_.full_potential()) {
@@ -374,14 +422,14 @@ void Potential::load()
     }
 
     if (!ctx_.full_potential()) {
-        sddk::HDF5_tree fout(storage_file_name, sddk::hdf5_access_t::read_only);
         for (int j = 0; j < ctx_.unit_cell().num_atoms(); j++) {
-            fout["unit_cell"]["atoms"][j].read("D_operator", ctx_.unit_cell().atom(j).d_mtrx());
+            fin["unit_cell"]["atoms"][j].read("D_operator", ctx_.unit_cell().atom(j).d_mtrx());
         }
     }
 }
 
-void Potential::update_atomic_potential()
+void
+Potential::update_atomic_potential()
 {
     for (int ic = 0; ic < unit_cell_.num_atom_symmetry_classes(); ic++) {
         int ia   = unit_cell_.atom_symmetry_class(ic).atom_id(0);
@@ -390,23 +438,22 @@ void Potential::update_atomic_potential()
         std::vector<double> veff(nmtp);
 
         for (int ir = 0; ir < nmtp; ir++) {
-            veff[ir] = y00 * effective_potential().f_mt<sddk::index_domain_t::global>(0, ir, ia);
+            veff[ir] = y00 * effective_potential().mt()[ia](0, ir);
         }
 
         unit_cell_.atom_symmetry_class(ic).set_spherical_potential(veff);
     }
 
     for (int ia = 0; ia < unit_cell_.num_atoms(); ia++) {
-        double* veff = &effective_potential().f_mt<sddk::index_domain_t::global>(0, 0, ia);
+        double* veff = &effective_potential().mt()[ia](0, 0);
 
         double* beff[] = {nullptr, nullptr, nullptr};
         for (int i = 0; i < ctx_.num_mag_dims(); i++) {
-            beff[i] = &effective_magnetic_field(i).f_mt<sddk::index_domain_t::global>(0, 0, ia);
+            beff[i] = &effective_magnetic_field(i).mt()[ia](0, 0);
         }
 
         unit_cell_.atom(ia).set_nonspherical_potential(veff, beff);
     }
 }
 
-}
-
+} // namespace sirius
