@@ -26,8 +26,10 @@
 #define __RADIAL_INTEGRALS_HPP__
 
 #include "unit_cell/unit_cell.hpp"
-#include "specfunc/sbessel.hpp"
-#include "utils/rte.hpp"
+#include "core/sf/sbessel.hpp"
+#include "core/rte/rte.hpp"
+#include "core/env/env.hpp"
+#include "core/ostream_tools.hpp"
 
 namespace sirius {
 
@@ -43,10 +45,10 @@ class Radial_integrals_base
     Radial_grid<double> grid_q_;
 
     /// Split index of q-points.
-    sddk::splindex<sddk::splindex_t::block> spl_q_;
+    splindex_block<> spl_q_;
 
     /// Array with integrals.
-    sddk::mdarray<Spline<double>, N> values_;
+    mdarray<Spline<double>, N> values_;
 
     /// Maximum length of the reciprocal wave-vector.
     double qmax_{0};
@@ -64,13 +66,14 @@ class Radial_integrals_base
         qmax_ = qmax__ + std::max(10.0, qmax__ * 0.1);
 
         grid_q_ = Radial_grid_lin<double>(static_cast<int>(np__ * qmax_), 0, qmax_);
-        spl_q_  = sddk::splindex<sddk::splindex_t::block>(grid_q_.num_points(), unit_cell_.comm().size(),
-                                                          unit_cell_.comm().rank());
+        spl_q_  = splindex_block<>(grid_q_.num_points(), n_blocks(unit_cell_.comm().size()),
+                                  block_id(unit_cell_.comm().rank()));
     }
 
     /// Get starting index iq and delta dq for the q-point on the linear grid.
     /** The following condition is satisfied: q = grid_q[iq] + dq */
-    inline std::pair<int, double> iqdq(double q__) const
+    inline std::pair<int, double>
+    iqdq(double q__) const
     {
         if (q__ > grid_q_.last()) {
             std::stringstream s;
@@ -91,18 +94,21 @@ class Radial_integrals_base
 
     /// Return value of the radial integral with specific indices.
     template <typename... Args>
-    inline double value(Args... args, double q__) const
+    inline double
+    value(Args... args, double q__) const
     {
         auto idx = iqdq(q__);
         return values_(args...)(idx.first, idx.second);
     }
 
-    inline int nq() const
+    inline int
+    nq() const
     {
         return grid_q_.num_points();
     }
 
-    inline double qmax() const
+    inline double
+    qmax() const
     {
         return qmax_;
     }
@@ -117,14 +123,15 @@ class Radial_integrals_atomic_wf : public Radial_integrals_base<2>
     /// Callback function to compute radial integrals using the host code.
     std::function<void(int, double, double*, int)> atomic_wfc_callback_{nullptr};
     /// Return radial basis index for a given atom type.
-    std::function<sirius::experimental::radial_functions_index const&(int)> indexr_;
+    std::function<radial_functions_index const&(int)> indexr_;
     /// Generate radial integrals.
-    void generate(std::function<Spline<double> const&(int, int)> fl__);
+    void
+    generate(std::function<Spline<double> const&(int, int)> fl__);
 
   public:
     /// Constructor.
     Radial_integrals_atomic_wf(Unit_cell const& unit_cell__, double qmax__, int np__,
-                               std::function<sirius::experimental::radial_functions_index const&(int)> indexr__,
+                               std::function<radial_functions_index const&(int)> indexr__,
                                std::function<Spline<double> const&(int, int)> fl__,
                                std::function<void(int, double, double*, int)> atomic_wfc_callback__)
         : Radial_integrals_base<2>(unit_cell__, qmax__, np__)
@@ -137,26 +144,28 @@ class Radial_integrals_atomic_wf : public Radial_integrals_base<2>
                 nrf_max = std::max(nrf_max, static_cast<int>(indexr_(iat).size()));
             }
 
-            values_ = sddk::mdarray<Spline<double>, 2>(nrf_max, unit_cell_.num_atom_types());
+            values_ = mdarray<Spline<double>, 2>({nrf_max, unit_cell_.num_atom_types()});
 
             generate(fl__);
         }
     }
 
     /// Retrieve a value for a given orbital of an atom type.
-    inline Spline<double> const& values(int iwf__, int iat__) const
+    inline Spline<double> const&
+    values(int iwf__, int iat__) const
     {
         return values_(iwf__, iat__);
     }
 
     /// Get all values for a given atom type and q-point.
-    inline sddk::mdarray<double, 1> values(int iat__, double q__) const
+    inline mdarray<double, 1>
+    values(int iat__, double q__) const
     {
         auto idx        = iqdq(q__);
         auto& atom_type = unit_cell_.atom_type(iat__);
         int nrf         = indexr_(atom_type.id()).size();
 
-        sddk::mdarray<double, 1> val(nrf);
+        mdarray<double, 1> val({nrf});
 
         if (atomic_wfc_callback_ == nullptr) {
             for (int i = 0; i < nrf; i++) {
@@ -176,7 +185,8 @@ class Radial_integrals_aug : public Radial_integrals_base<3>
   private:
     std::function<void(int, double, double*, int, int)> ri_callback_{nullptr};
 
-    void generate();
+    void
+    generate();
 
   public:
     Radial_integrals_aug(Unit_cell const& unit_cell__, double qmax__, int np__,
@@ -188,20 +198,20 @@ class Radial_integrals_aug : public Radial_integrals_base<3>
             int nmax = unit_cell_.max_mt_radial_basis_size();
             int lmax = unit_cell_.lmax();
 
-            values_ =
-                sddk::mdarray<Spline<double>, 3>(nmax * (nmax + 1) / 2, 2 * lmax + 1, unit_cell_.num_atom_types());
+            values_ = mdarray<Spline<double>, 3>({nmax * (nmax + 1) / 2, 2 * lmax + 1, unit_cell_.num_atom_types()});
 
             generate();
         }
     }
 
-    inline sddk::mdarray<double, 2> values(int iat__, double q__) const
+    inline mdarray<double, 2>
+    values(int iat__, double q__) const
     {
         auto& atom_type = unit_cell_.atom_type(iat__);
         int lmax        = atom_type.indexr().lmax();
         int nbrf        = atom_type.mt_radial_basis_size();
 
-        sddk::mdarray<double, 2> val(nbrf * (nbrf + 1) / 2, 2 * lmax + 1);
+        mdarray<double, 2> val({nbrf * (nbrf + 1) / 2, 2 * lmax + 1});
 
         if (ri_callback_ == nullptr) {
             auto idx = iqdq(q__);
@@ -222,7 +232,8 @@ class Radial_integrals_rho_pseudo : public Radial_integrals_base<1>
 {
   private:
     std::function<void(int, int, double*, double*)> ri_callback_{nullptr};
-    void generate();
+    void
+    generate();
 
   public:
     Radial_integrals_rho_pseudo(Unit_cell const& unit_cell__, double qmax__, int np__,
@@ -231,33 +242,36 @@ class Radial_integrals_rho_pseudo : public Radial_integrals_base<1>
         , ri_callback_(ri_callback__)
     {
         if (ri_callback__ == nullptr) {
-            values_ = sddk::mdarray<Spline<double>, 1>(unit_cell_.num_atom_types());
+            values_ = mdarray<Spline<double>, 1>({unit_cell_.num_atom_types()});
             generate();
 
-            if (unit_cell_.parameters().cfg().control().print_checksum() && unit_cell_.comm().rank() == 0) {
+            auto pcs = env::print_checksum();
+
+            if (pcs && unit_cell_.comm().rank() == 0) {
                 double cs{0};
                 for (int iat = 0; iat < unit_cell_.num_atom_types(); iat++) {
                     for (int iq = 0; iq < grid_q_.num_points(); iq++) {
                         cs += values_(iat)(iq);
                     }
                 }
-                utils::print_checksum("Radial_integrals_rho_pseudo", cs, std::cout);
+                print_checksum("Radial_integrals_rho_pseudo", cs, std::cout);
             }
         }
     }
 
     /// Compute all values of the raial integrals.
-    inline sddk::mdarray<double, 2> values(std::vector<double>& q__, mpi::Communicator const& comm__) const
+    inline auto
+    values(std::vector<double>& q__, mpi::Communicator const& comm__) const
     {
         int nq = static_cast<int>(q__.size());
-        sddk::splindex<sddk::splindex_t::block> splq(nq, comm__.size(), comm__.rank());
-        sddk::mdarray<double, 2> result(nq, unit_cell_.num_atom_types());
+        splindex_block<> splq(nq, n_blocks(comm__.size()), block_id(comm__.rank()));
+        mdarray<double, 2> result({nq, unit_cell_.num_atom_types()});
         result.zero();
         for (int iat = 0; iat < unit_cell_.num_atom_types(); iat++) {
             if (!unit_cell_.atom_type(iat).ps_total_charge_density().empty()) {
                 #pragma omp parallel for
                 for (int iqloc = 0; iqloc < splq.local_size(); iqloc++) {
-                    int iq = splq[iqloc];
+                    auto iq = splq.global_index(iqloc);
                     if (ri_callback_) {
                         ri_callback_(iat + 1, 1, &q__[iq], &result(iq, iat));
                     } else {
@@ -276,7 +290,8 @@ class Radial_integrals_rho_core_pseudo : public Radial_integrals_base<1>
 {
   private:
     std::function<void(int, int, double*, double*)> ri_callback_{nullptr};
-    void generate();
+    void
+    generate();
 
   public:
     Radial_integrals_rho_core_pseudo(Unit_cell const& unit_cell__, double qmax__, int np__,
@@ -285,23 +300,24 @@ class Radial_integrals_rho_core_pseudo : public Radial_integrals_base<1>
         , ri_callback_(ri_callback__)
     {
         if (ri_callback_ == nullptr) {
-            values_ = sddk::mdarray<Spline<double>, 1>(unit_cell_.num_atom_types());
+            values_ = mdarray<Spline<double>, 1>({unit_cell_.num_atom_types()});
             generate();
         }
     }
 
     /// Compute all values of the raial integrals.
-    inline sddk::mdarray<double, 2> values(std::vector<double>& q__, mpi::Communicator const& comm__) const
+    inline auto
+    values(std::vector<double>& q__, mpi::Communicator const& comm__) const
     {
         int nq = static_cast<int>(q__.size());
-        sddk::splindex<sddk::splindex_t::block> splq(nq, comm__.size(), comm__.rank());
-        sddk::mdarray<double, 2> result(nq, unit_cell_.num_atom_types());
+        splindex_block<> splq(nq, n_blocks(comm__.size()), block_id(comm__.rank()));
+        mdarray<double, 2> result({nq, unit_cell_.num_atom_types()});
         result.zero();
         for (int iat = 0; iat < unit_cell_.num_atom_types(); iat++) {
             if (!unit_cell_.atom_type(iat).ps_core_charge_density().empty()) {
                 #pragma omp parallel for
                 for (int iqloc = 0; iqloc < splq.local_size(); iqloc++) {
-                    int iq = splq[iqloc];
+                    auto iq = splq.global_index(iqloc);
                     if (ri_callback_) {
                         ri_callback_(iat + 1, 1, &q__[iq], &result(iq, iat));
                     } else {
@@ -324,7 +340,8 @@ class Radial_integrals_beta : public Radial_integrals_base<2>
     std::function<void(int, double, double*, int)> ri_callback_{nullptr};
 
     /// Generate radial integrals on the q-grid.
-    void generate();
+    void
+    generate();
 
   public:
     Radial_integrals_beta(Unit_cell const& unit_cell__, double qmax__, int np__,
@@ -334,16 +351,17 @@ class Radial_integrals_beta : public Radial_integrals_base<2>
     {
         if (ri_callback_ == nullptr) {
             /* create space for <j_l(qr)|beta> or <d j_l(qr) / dq|beta> radial integrals */
-            values_ = sddk::mdarray<Spline<double>, 2>(unit_cell_.max_mt_radial_basis_size(), unit_cell_.num_atom_types());
+            values_ = mdarray<Spline<double>, 2>({unit_cell_.max_mt_radial_basis_size(), unit_cell_.num_atom_types()});
             generate();
         }
     }
 
     /// Get all values for a given atom type and q-point.
-    inline sddk::mdarray<double, 1> values(int iat__, double q__) const
+    inline mdarray<double, 1>
+    values(int iat__, double q__) const
     {
         auto& atom_type = unit_cell_.atom_type(iat__);
-        sddk::mdarray<double, 1> val(atom_type.mt_radial_basis_size());
+        mdarray<double, 1> val({atom_type.mt_radial_basis_size()});
         if (ri_callback_ == nullptr) {
             auto idx = iqdq(q__);
             for (int i = 0; i < atom_type.mt_radial_basis_size(); i++) {
@@ -361,7 +379,8 @@ class Radial_integrals_vloc : public Radial_integrals_base<1>
 {
   private:
     std::function<void(int, int, double*, double*)> ri_callback_{nullptr};
-    void generate();
+    void
+    generate();
 
   public:
     Radial_integrals_vloc(Unit_cell const& unit_cell__, double qmax__, int np__,
@@ -370,13 +389,14 @@ class Radial_integrals_vloc : public Radial_integrals_base<1>
         , ri_callback_(ri_callback__)
     {
         if (ri_callback_ == nullptr) {
-            values_ = sddk::mdarray<Spline<double>, 1>(unit_cell_.num_atom_types());
+            values_ = mdarray<Spline<double>, 1>({unit_cell_.num_atom_types()});
             generate();
         }
     }
 
     /// Special implementation to recover the true radial integral value.
-    inline double value(int iat__, double q__) const
+    inline double
+    value(int iat__, double q__) const
     {
         if (unit_cell_.atom_type(iat__).local_potential().empty()) {
             return 0;
@@ -402,17 +422,18 @@ class Radial_integrals_vloc : public Radial_integrals_base<1>
     }
 
     /// Compute all values of the raial integrals.
-    inline sddk::mdarray<double, 2> values(std::vector<double>& q__, mpi::Communicator const& comm__) const
+    inline auto
+    values(std::vector<double>& q__, mpi::Communicator const& comm__) const
     {
         int nq = static_cast<int>(q__.size());
-        sddk::splindex<sddk::splindex_t::block> splq(nq, comm__.size(), comm__.rank());
-        sddk::mdarray<double, 2> result(nq, unit_cell_.num_atom_types());
+        splindex_block<> splq(nq, n_blocks(comm__.size()), block_id(comm__.rank()));
+        mdarray<double, 2> result({nq, unit_cell_.num_atom_types()});
         result.zero();
         for (int iat = 0; iat < unit_cell_.num_atom_types(); iat++) {
             if (!unit_cell_.atom_type(iat).local_potential().empty()) {
                 #pragma omp parallel for
                 for (int iqloc = 0; iqloc < splq.local_size(); iqloc++) {
-                    int iq = splq[iqloc];
+                    auto iq = splq.global_index(iqloc);
                     if (ri_callback_) {
                         ri_callback_(iat + 1, 1, &q__[iq], &result(iq, iat));
                     } else {
@@ -429,18 +450,20 @@ class Radial_integrals_vloc : public Radial_integrals_base<1>
 class Radial_integrals_rho_free_atom : public Radial_integrals_base<1>
 {
   private:
-    void generate();
+    void
+    generate();
 
   public:
     Radial_integrals_rho_free_atom(Unit_cell const& unit_cell__, double qmax__, int np__)
         : Radial_integrals_base<1>(unit_cell__, qmax__, np__)
     {
-        values_ = sddk::mdarray<Spline<double>, 1>(unit_cell_.num_atom_types());
+        values_ = mdarray<Spline<double>, 1>({unit_cell_.num_atom_types()});
         generate();
     }
 
     /// Special implementation to recover the true radial integral value.
-    inline double value(int iat__, double q__) const
+    inline double
+    value(int iat__, double q__) const
     {
         auto idx = iqdq(q__);
         if (std::abs(q__) < 1e-12) {
