@@ -29,6 +29,7 @@
 #include "spline.hpp"
 #include "core/constants.hpp"
 #include "core/typedefs.hpp"
+#include "core/rte/rte.hpp"
 
 namespace sirius {
 
@@ -174,7 +175,9 @@ class Radial_solver
             } else if (k__ == l__ + 1) {
                 kappa = -k__;
             } else {
-                throw std::runtime_error("wrong k");
+                std::stringstream s;
+                s << "wrong k : " << k__ << " for l = " << l__;
+                RTE_THROW(s);
             }
         }
 
@@ -291,21 +294,21 @@ class Radial_solver
 
             if (rel == relativity_t::none || rel == relativity_t::koelling_harmon || rel == relativity_t::zora) {
                 /* k0 = F(Y(x), x) */
-                pk[0] = 2 * M0 * q0 + p0 * xinv0;
+                pk[0] = 2 * M0 * q0 + p0 * xinv0 + chi_p0;
                 qk[0] = (v0 - enu__ + ll_half / M0 / std::pow(x0, 2)) * p0 - q0 * xinv0 + chi_q0;
 
                 /* k1 = F(Y(x) + k0 * h/2, x + h/2) */
-                pk[1] = 2 * M1 * (q0 + qk[0] * h_half) + (p0 + pk[0] * h_half) * xinv1;
+                pk[1] = 2 * M1 * (q0 + qk[0] * h_half) + (p0 + pk[0] * h_half) * xinv1 + chi_p1;
                 qk[1] = (v1 - enu__ + ll_half / M1 / std::pow(x1, 2)) * (p0 + pk[0] * h_half) -
                         (q0 + qk[0] * h_half) * xinv1 + chi_q1;
 
                 /* k2 = F(Y(x) + k1 * h/2, x + h/2) */
-                pk[2] = 2 * M1 * (q0 + qk[1] * h_half) + (p0 + pk[1] * h_half) * xinv1;
+                pk[2] = 2 * M1 * (q0 + qk[1] * h_half) + (p0 + pk[1] * h_half) * xinv1 + chi_p1;
                 qk[2] = (v1 - enu__ + ll_half / M1 / std::pow(x1, 2)) * (p0 + pk[1] * h_half) -
                         (q0 + qk[1] * h_half) * xinv1 + chi_q1;
 
                 /* k3 = F(Y(x) + k2 * h, x + h) */
-                pk[3] = 2 * M2 * (q0 + qk[2] * h) + (p0 + pk[2] * h) * xinv2;
+                pk[3] = 2 * M2 * (q0 + qk[2] * h) + (p0 + pk[2] * h) * xinv2 + chi_p2;
                 qk[3] = (v2 - enu__ + ll_half / M2 / std::pow(x2, 2)) * (p0 + pk[2] * h) - (q0 + qk[2] * h) * xinv2 +
                         chi_q2;
             }
@@ -421,6 +424,19 @@ class Radial_solver
             }
         }
 
+        /* normalize solution */
+        if (false) {
+            Spline<double> s(radial_grid_);
+            for (int i = 0; i < nr; i++) {
+                s(i) = std::pow(p__[i], 2);
+            }
+            auto norm = 1.0 / std::sqrt(s.interpolate().integrate(0));
+            for (int i = 0; i < nr; i++) {
+                p__[i] *= norm;
+                q__[i] *= norm;
+            }
+        }
+
         for (int i = 0; i < nr; i++) {
             if (rel == relativity_t::none || rel == relativity_t::koelling_harmon || rel == relativity_t::zora) {
                 double V  = ve_(i) - zn_ * radial_grid_.x_inv(i);
@@ -428,7 +444,7 @@ class Radial_solver
                 double v1 = ll_half / M / std::pow(radial_grid_[i], 2);
 
                 /* P' = 2MQ + \frac{P}{r} */
-                dpdr__[i] = 2 * M * q__[i] + p__[i] * radial_grid_.x_inv(i);
+                dpdr__[i] = 2 * M * q__[i] + p__[i] * radial_grid_.x_inv(i) + chi_p__(i);
 
                 /* Q' = (V - E + \frac{\ell(\ell + 1)}{2 M r^2}) P - \frac{Q}{r} */
                 dqdr__[i] = (V - enu__ + v1) * p__[i] - q__[i] * radial_grid_.x_inv(i) + chi_q__(i);
@@ -674,7 +690,6 @@ class Radial_solver
                     for (int i = 0; i < nr; i++) {
                         chi_q(i) = -j * p[j - 1][i];
                     }
-                    chi_q.interpolate();
                 } else if (rel__ == relativity_t::koelling_harmon) {
                     double sq_alpha = std::pow(speed_of_light, -2);
                     double ll_half  = l__ * (l__ + 1) / 2.0;
@@ -686,8 +701,20 @@ class Radial_solver
                             chi_p(i) = sq_alpha * q[j - 1][i];
                             chi_q(i) = -p[j - 1][i] * (1 + 0.5 * sq_alpha * ll_half / std::pow(M * x, 2));
                         }
+                    } else if (j == 2) {
+                        for (int i = 0; i < nr; i++) {
+                            double x = radial_grid_[i];
+                            double V = ve_(i) - zn_ * radial_grid_.x_inv(i);
+                            double M = 1 + 0.5 * sq_alpha * (enu__ - V);
+                            chi_p(i) = 2 * sq_alpha * q[j - 1][i];
+                            chi_q(i) = -2 * p[j - 1][i] * (1 + 0.5 * sq_alpha * ll_half / std::pow(M * x, 2)) +
+                                       p[j - 2][i] * (0.5 * ll_half * std::pow(sq_alpha, 2) / std::pow(M * x, 2) / M);
+                        }
                     } else {
-                        throw std::runtime_error("not implemented");
+                        std::stringstream s;
+                        s << "energy derivative of the order " << j
+                          << " is not implemented for Koelling-Harmon radial solver";
+                        RTE_THROW(s);
                     }
                 } else if (rel__ == relativity_t::iora) {
                     double sq_alpha = std::pow(speed_of_light, -2);
@@ -712,13 +739,15 @@ class Radial_solver
                             chi_q(i) = -p[j - 1][i] * 2 * (1 + 0.5 * sq_alpha * ll_half / std::pow(M * x, 2));
                         }
                     } else {
-                        throw std::runtime_error("not implemented");
+                        std::stringstream s;
+                        s << "energy derivative of the order " << j << " is not implemented for IORA radial solver";
+                        RTE_THROW(s);
                     }
-                    chi_p.interpolate();
-                    chi_q.interpolate();
                 } else {
-                    throw std::runtime_error("not implemented");
+                    RTE_THROW("unsupported relativity type");
                 }
+                chi_p.interpolate();
+                chi_q.interpolate();
             }
 
             switch (rel__) {
@@ -743,7 +772,7 @@ class Radial_solver
                     break;
                 }
                 default: {
-                    throw std::runtime_error("not implemented");
+                    RTE_THROW("unsupported relativity type");
                 }
             }
         }
@@ -904,7 +933,7 @@ class Bound_state : public Radial_solver
                     break;
                 }
                 default: {
-                    throw std::runtime_error("not implemented");
+                    RTE_THROW("unsupported relativity type");
                 }
             }
 
@@ -923,7 +952,7 @@ class Bound_state : public Radial_solver
             std::stringstream s;
             s << "enu is not converged for n = " << n_ << " and l = " << l_ << std::endl
               << "enu = " << enu_ << ", denu = " << denu;
-            throw std::runtime_error(s.str());
+            RTE_THROW(s);
         }
 
         /* compute r * u'(r) */
@@ -997,7 +1026,7 @@ class Bound_state : public Radial_solver
               << "l = " << l_ << std::endl
               << "enu = " << enu_ << std::endl
               << "wrong number of nodes : " << nn << " instead of " << (n_ - l_ - 1);
-            throw std::runtime_error(s.str());
+            RTE_THROW(s);
         }
 
         for (int i = 0; i < np - 1; i++) {
@@ -1114,7 +1143,7 @@ class Enu_finder : public Radial_solver
                     break;
                 }
                 default: {
-                    throw std::runtime_error("not implemented");
+                    RTE_THROW("unsupported relativity type");
                 }
             }
             nnd -= (n_ - l_ - 1);
@@ -1169,7 +1198,7 @@ class Enu_finder : public Radial_solver
                     break;
                 }
                 default: {
-                    throw std::runtime_error("not implemented");
+                    RTE_THROW("unsupported relativity type");
                 }
             }
             if (surface_deriv() * sd <= 0) {
@@ -1202,7 +1231,7 @@ class Enu_finder : public Radial_solver
                     break;
                 }
                 default: {
-                    throw std::runtime_error("not implemented");
+                    RTE_THROW("unsupported relativity type");
                 }
             }
             /* derivative at the boundary */
@@ -1239,7 +1268,7 @@ class Enu_finder : public Radial_solver
                 break;
             }
             default: {
-                throw std::runtime_error("not implemented");
+                RTE_THROW("unsupported relativity type");
             }
         }
 
@@ -1258,7 +1287,7 @@ class Enu_finder : public Radial_solver
               << "etop: " << etop_ << " ebot: " << ebot_ << std::endl
               << "initial surface derivative: " << sd;
 
-            throw std::runtime_error(s.str());
+            RTE_THROW(s);
         }
 
         enu_ = (ebot_ + etop_) / 2.0;
