@@ -143,7 +143,6 @@ Atom_type::init()
 
     if (!parameters_.full_potential()) {
         RTE_ASSERT(mt_radial_basis_size() == num_beta_radial_functions());
-        // RTE_ASSERT(lmax_beta() == indexr1().lmax());
     }
 
     /* get number of valence electrons */
@@ -719,12 +718,17 @@ Atom_type::read_input(std::string const& str__)
     }
 
     if (parameters_.full_potential()) {
-        name_           = parser["name"].get<std::string>();
-        symbol_         = parser["symbol"].get<std::string>();
-        mass_           = parser["mass"].get<double>();
-        zn_             = parser["number"].get<int>();
-        double r0       = parser["rmin"].get<double>();
-        double R        = parser["rmt"].get<double>();
+        name_     = parser["name"].get<std::string>();
+        symbol_   = parser["symbol"].get<std::string>();
+        mass_     = parser["mass"].get<double>();
+        zn_       = parser["number"].get<int>();
+        double r0 = parser["rmin"].get<double>();
+        double R  = parser["rmt"].get<double>();
+        try { /* overwrite the muffin-tin radius with the value from the inpupt */
+            R = parameters_.cfg().unit_cell().atom_type_rmt(label_);
+        } catch (...) {
+        }
+
         int nmtp        = parser["nrmt"].get<int>();
         this->lmax_apw_ = parser.value("lmax_apw", this->lmax_apw_);
 
